{$O-}
program CelPack;

uses dos,crt;


type
     DataPtr = ^byte;

{**************************************************************************}
{                                                                          }
{  MSGDATAHEADER Record                                                    }
{                                                                          }
{  The MsgDataHeader is a very important record, but is never used in of   }
{  itself.  For all CelerityText data entries (messages, file descriptions,}
{  menus, language file entries, email, etc), there is an index file and   }
{  a data file.  The index file is always a typed file (see other sections }
{  for details on these record formats), but the data file is dynamic.     }
{  This is where the MsgDataHeader comes in.  At the very beginning of a   }
{  data segment in a CelerityText data file is a special 9-byte header.    }
{  This header is the MsgDataHeader, which contains some important details }
{  about the data that follows.  The IndexOffset refers back to the index  }
{  file and contains the index number of the record which is associated    }
{  with the data segment.  The size tells Celerity how many bytes are in   }
{  the data segment, and CompType refers to the "compression format",      }
{  which is currently unused.                                              }
{                                                                          }
{**************************************************************************}
type	MsgDataHeader = Record
		  IndexOffset	: longint;	{Location of index entry}
		  Size				: longint;	{Size of message}
		  CompType		: byte;			{Compression type - unused in 2.0}
		  end;

{**************************************************************************}
{                                                                          }
{  MSGINDEXREC Record                                                      }
{                                                                          }
{  The MsgIndexRecord is vital to the Celerity v2.00+ messaging system.    }
{  The *.IDX index files in the message bases and MAIL.IDX file are typed  }
{  files based on this record.  It contains all pertinent information      }
{  regarding messages on a Celerity system.                                }
{                                                                          }
{**************************************************************************}
type MsgIndexRec = record     {*.IDX format for Celerity v2.00}
		 Title		:	string[71];	{Title of the message}
		 Author		:	string[35];   {Handle of message author}
		 AuthorAddr:string[127];	  {This is the address of the author}
     AuthorNote: string[30];  {Authors User Note}
		 LeftTo		: string[35];   {Handle of addressee}
		 LeftAddr	:	string[127];	  {Address of the recipient}
		 ForwardBy	: string[35]; {Handle of the user who forwarded the message}
		 ForwardAddr:string[127];	{This is the address of the fowarder}
		 When			: longint;		  {When message was written in PACKTIME}

		 TimesRead	: word;				{Message read how many times??}
		 Return		: boolean;		  {Return Receipt requested}
		 Anon			: boolean;		  {Anonymous message}
		 FileName	: string[12];   {Name of attached file}

		 Offset		: longint;		  {position of data in data file}
		 Size			: longint;		  {size of message in bytes}

		 {flags....}
		 Deleted		: boolean;		{message is deleted}
     Validated : boolean;     {message validatation state}
     Personal  : boolean;     {}

		 {Thread Info}
		 Origin,             		  {Original post of thread}
		 Parent,									{Parent of this message}
     PrevSibling,             {Prev reply to the original message. (Sibling)}
		 NextSibling,						  {Next reply to the original message (sibling)}
		 Child			: longint;		{First reply to THIS post.. eldest "child"}

     MsgRefId  : longint;     {Reference to this post}
     FidoToss,
     Toss2,
     Toss3,                   {Flags for third-party utilities}
     ExtTossed : boolean;     {External utility tossed}

     NetStatus : byte;
     NetName   : string[15];
     NetIdent  : string[30];  {Name for network}
     AssortedCrap:array[1..65] of char;
		 end;


{**************************************************************************}
{                                                                          }
{  MSGBASEREC Record                                                       }
{                                                                          }
{  The MsgBaseRec record contains the format for Celerity's message base   }
{  configuration files (*.BSE in the conference directory).  It contains   }
{  all needed information about the actual message base, its configuration,}
{  restrictions, and so forth.                                             }
{                                                                          }
{**************************************************************************}
type MsgBaseRec=record {*.BSE format for Celerity v2.00}
		 Name:string[30];                {Name of sub-board}
		 QwkName:string[15];             {Name used in QWK readers}
		 AccessCode:string[15];          {Command to get to sub board}
     AttachedPath,             {Path to attached files}
		 DataPath,                 {File name of the Sub Data}
     IndexPath,                {File name of the Sub Index}
     QscanPath : pathstr;      {File name of quickscan data}
		 OriginLine:string[80];          {This sub's origin line}
		 BoardMsgLimit:integer;    {Maximum number of messages on sub  }
     CurrentMsgCount:integer;  {every time post made, inc. this}
		 NetType:byte;             {0=local, 1=celnet, 2=fido, 3=mercnet, 4=qwk}
		 NetAddress:string[15];          {network address use}
		 NetName:string[15];            {network address use}
		 BaseNews:string[80];            {path to message base news/intro file}
		 Compression:byte;         {0=uncompressed - default}
     PostSizeLimit:integer;    {0=no limit.  A byte was too small.}
     FileCredit:integer;       {bytes per credit point.      }
                               {If negative, points per post.}
		 RealNames,                {Use real names on sub}
		 AllowAnsi,                {Allow ANSI                    }
		 AllowNaplps,              {Allow NAPLPS                  }
		 AllowAnony,               {Allow Anonymous posts         }
		 AllowFileAttach,          {Allow attached files          }
		 AllowPrivateMsg,          {Allow receiver-only messages  }
		 AllowUpload,              {Allow post uploads            }
		 AllowDnload,              {Allow post downloads          }
     RequireValidate,          {Require Post Validation       }
     TimeStopped,              {Time doesn't count when in sub}
		 option1,
		 option2,
		 option3,
		 option4,
		 option5 :boolean;

     CurMsgRefID: longint;    {Last message id on this sub}
     Index      : longint;    {Pointer to the records loc in data file  }
                              {This needs to be updated when subs change}
     QwkConfNum : Word;
     Space: String[158];   {Spare work space}
		 end;


{**************************************************************************}
{                                                                          }
{  FILEREC Record                                                          }
{                                                                          }
{  The FileRec record (UdRec in 1.42 and previously) is the record used for}
{  all file listings in Celerity v2.00+ transfer sections.  All Celerity   }
{  *.DIR files are made up of this record, and have a corresponding .DES   }
{  data file (CelerityText with MsgDataHeader - see above) for description }
{  data.  The DescOffset and DescSize refer to this .DES file.             }
{                                                                          }
{**************************************************************************}
type FileRec=record                 { *.DIR for version 2.00+ 7/14/93}
       sentby:string[30];           { Who uploaded the file}
       when,                        { When was it uploaded - PACKTIME}
       whenrated:longint;           { Show up on newscans since.. PACKTIME}
       filename,                    { DOS filename}
       dlpw:string[15];             { Password to download it}
       path:string[50];             { Full pathname pointing to location}
       points:integer;              { Cost to download}
       filesize:longint;            { Size (in bytes)}
       SystemNote:string[44];       { Contains system messages re:viruses etc}
       DescOffset,                  { Offset of description data}
       DescSize,                    { Size of description}
       ExtDescOffset,               { Offset of extended description}
       ExtDescSize:longint;         { Size of extended description}
       DescLength:byte;             { Length (# of lines) of description}
       DateMoved:longint;           { Date the file was last moved}
       Unused:string[14];           { unused with 2.0}
       downloaded:integer;          { Number of times file downloaded}
       newfile,                     { Unvalidated file}
       FreeFile:boolean;            { File is a free download}
       progname:string[20];         { This is the full/program name of the program}
       ofwhat1,ofwhat2:byte;        { These are for "disk x of y"}
       ptsgiven:integer;            { How many validation points awarded}
       private:string[30];          { Who is a private file for}
     end;


type TextIndexRec=record    { Record to point to text location }
     Size,
     Offset:Longint;
     end;


var dir,moo:string;
    OldDesc,NewDesc:File of char;
    DirInfo: SearchRec;
    D,Name,Ext:namestr;
    Counter:integer;
    Data:DataPtr;


    OldAttr:word;

    PackDesc,
    PackMail,
    PackBase,
    PackLang:boolean;
    unew:file of FileRec;
    newu:FileRec;
    TotalSize,Saved:longint;
    AreaTotal,AreaSaved:longint;
    Numfiles:integer;
    DelDate:integer;
    filename:string;


 const
	{Constants for File Locking}
                        {The +2 allows opening for Read/Write}
	RW				=	 0+2;	    {Fully Open}
	Locked		= 16+2;	    {Read/Write Denied to other programs}
	ReadOnly	=	32+2;	    {Write Access Denied}
	WriteOnly	=	48+2;	    {Read Access Denied}

	Denied		=	 5;	      {File Locked Error}



function packtime (var dt:datetime):longint;
var l:longint;
begin
  dos.packtime (dt,l);
  packtime:=l
end;

function now:longint;
var dt:datetime;
    t:word;
    l:longint;
begin
  gettime (dt.hour,dt.min,dt.sec,t);
  getdate (dt.year,dt.month,dt.day,t);
  l:=packtime(dt);
  now:=l
end;

function NetReset(var af; size:word; mode:byte):byte;
var
	cnt,
	initmode: byte;
	result	: word;
	f				: file absolute af;
begin
	initmode:=filemode;	{store the original filemode}

	filemode:=mode;			{Set the desired mode}

	if size=0 then		{Branch for Record Size}
		reset(f)
	else
		reset(f,size);

	result:=ioresult;

	cnt:=0;

	while (result=Denied) and (cnt < 200) do begin	{Approx 2 Seconds...}
		if size=0 then		{Branch for Record Size}
			reset(f)
		else
			reset(f,size);

		result:=ioresult;
		inc(cnt);					{Inc Loop Counter}
		delay(10);				{Delay 1/100 sec }

		end;

	FileMode:=InitMode;		{Restore original FileMode}

	NetReset:=Result;			{Return the result code}

end;


procedure SaveMsgData(var Index:MsgIndexRec; IndexLoc: longint; Data:dataPtr; FN: pathstr);
{ Save the actual message data to the end of the message data file }
{ IndexLoc needs to be passed in so that the data header can refer back}
var
	datafile	: file;
	offset		: longint;
	header		: MsgDataHeader;
	Result		: byte;

begin
	assign(datafile,FN);
	Result:=NetReset(datafile,1,Locked);

  if result=2 then begin
    writeln(^M'Creating ',fn,^M);
    readln;
    Result:=NetReset(datafile,1,Locked);
		rewrite(Datafile,1);
    close(DataFile);
    result:=0;
    end;

	case result of

		0:begin
			offset:=filesize(datafile);		{Get Filesize}
			seek(datafile,offset);				{Seek to EOF}

			Index.offset:=offset;         {Pos of the Data in data file}

			With Header do begin
				IndexOffset:=IndexLoc;    {This is the Loc of the INDEX}
				Size:=Index.Size;         {Above needs to be RefNumId}
				CompType:=0;
				end;

			blockwrite(datafile,header,sizeof(header) );

			blockwrite(datafile,data^,index.size);
			end;
		5:writeln('File Busy: ',FN);							{File Locked - Repeat??}

		else begin                                {Create the file first}
    end;
	end; {case}

	close(datafile);
end;


procedure ReadMsgData(var offset:longint;var size:word;var data:dataPtr; FN: pathstr);
var
	datafile: file;
	Header	: MsgDataHeader;
	Result	: byte;

begin
	{$I-}
	assign(datafile, FN );
	Result:=NetReset(datafile,1,RW);
{  writeln;
  writeln('Result:',result,'  ',fn);
}
	if Result = 0 then begin
    if offset>=filesize(datafile) then begin
      offset:=0;
      size:=0;
      close(datafile);
      exit;
      end;
		seek(datafile,offset);
		blockread(datafile,Header,Sizeof(Header) );
		blockread(datafile,data^,size);
		end
	else begin
    end;

	close(datafile);
end;

procedure CopyData(Size:word; var Offset:longint; FromFile,ToFile:Pathstr);
var Data:DataPtr;
    Dummy:MsgIndexRec;
    IndexLoc:longint;
begin
  getmem(Data,Size);
{  write('Copying ',size,' from ',offset,'/',fromfile);}
  ReadMsgData(Offset,Size,Data,FromFile);
  Dummy.Size:=size;

  SaveMsgData(Dummy, 0, Data, ToFile);
  Offset:=Dummy.OffSet;
{  writeln('  to ',offset,'/',Tofile);
  writeln;}
  FreeMem(data,size);
end;



function strr (i:integer):string;
var b:string;
begin
  str (i,b);
  strr:=b
end;

procedure DoStatus;
var f:text;
    cnt:integer;
begin
  cnt:=ioresult;
  assign(f,'');
  rewrite(f);
  cnt:=ioresult;
  window(1,1,80,25);
  textcolor(7);
  writeln('Processing all selected data files in '+dir);
  writeln('Text Data Bytes Processed: ',totalsize);
  writeln('Text Data Bytes ReWritten: ',Saved);
  writeln('Text Data Bytes Saved    : ',TotalSize-Saved);
  writeln('Packing File             : ',dirinfo.name+'         ',deldate);
  randomize;
  for cnt:=1 to 79 do begin
    textcolor(2);
    write('-');
    end;
  writeln;
  textcolor(7);
  window(1,7,80,25);

end;



  Procedure DoPackBase(NameOfBase:string;DeleteOld:integer);
  var
    NewIndex,
	  OldIndex     : file of MsgIndexRec;
	  OldData,NewData     : file;
	  x,
    numitems  :longint;
	  irec      : MsgIndexRec;
    total,
    deleted   :word;
    result    :byte;
    D,N,E     :string;
    OldDAttr,
    OldIAttr  :Word;

	  procedure ReadIndex(num: longint; var base: MsgIndexRec);
	  begin
		  Result:=NetReset(OldIndex,sizeof(MsgIndexRec),RW);
		  seek(OldIndex,num);
		  read(OldIndex,base);
		  close(OldIndex);
	  end;


	  begin
      total:=0;
      deleted:=0;
      DoStatus;
      fsplit(NameOfBase,D,N,E);
		  assign(OldData,d+n+'.dat');
  	  assign(OldIndex,d+n+'.IDX');
      GetFAttr(OldData,OldDAttr);
      GetFAttr(OldData,OldIAttr);
		  assign(NewData,d+'temp$.dat');
		  assign(NewIndex,d+'temp$.idx');
      erase(NewData);
      x:=ioresult;
      erase(NewIndex);
      x:=ioresult;

      result:=NetReset(OldIndex,sizeof(MsgIndexRec),Locked);
      numitems:=filesize(OldIndex);
		  close(OldIndex);

      if numitems<4 then begin
        exit;
        end;

		  Result:=NetReset(OldData,1, Locked);
      TotalSize:=TotalSize+FileSize(OldData);
      close(OldData);

		  rewrite(NewData,1);
		  rewrite(NewIndex);
      close(NewData);

      total:=numitems;
		  for x:=0 to numitems-1 do begin
        ReadIndex(x,irec);
        gotoxy(1,1);
        write(x,' Reindexing ',irec.title);
        clreol;
			  if (not irec.deleted) and (irec.leftto<>'') and (irec.Author<>'') and
          ((DeleteOld<1) or (irec.When>(Now-(65536*DeleteOld)))) then begin
          CopyData(irec.size,irec.Offset,d+n+'.dat',d+'temp$.dat');
				  write(Newindex,irec);
				  end
			  else begin
				  inc(deleted);
				  end;
			  end;


      close(OldIndex);
      close(OldData);
		  erase(OldData);
      erase(OldIndex);
      close(NewIndex);
      close(NewData);

		  rename(NewIndex,d+n+'.idx');
		  rename(NewData,d+n+'.dat');

      reset(NewData,1);
      saved:=saved+filesize(NewData);
      close(NewData);

      SetFAttr(NewIndex,OldIAttr);
      SetFAttr(NewData,OldDAttr);
      x:=ioresult;
      Gotoxy(1,19);
      writeln(N+'.IDX: Items Erased : ',deleted,' from a total of ',total,'   ',
           total-deleted,' remaining.');
      DoStatus;
	  end;

procedure DoPackLang(LangName:string;dsp:boolean);
var
   OldLNI,
   NewLNI:file of TextIndexRec;
   OldLNG,
   NewLNG:file;

   cnt:integer;
   Index:TextIndexRec;
   D,N,E:string;
   OldDAttr,
   OldIAttr: word;
   SizeOfData:longint;
   SizeNew:longint;
   SizeTest:longint;
begin
  fsplit(LangName,D,N,E);
  sizetest:=0;

  if dsp then begin
    assign(OldLNI,D+N+'.DSI');
    assign(NewLNI,D+'Temp$.DSI');
    assign(OldLng,D+N+'.DSP');
    assign(NewLNG,D+'Temp$.DSP');
  end else begin
    assign(OldLNI,D+N+'.LNI');
    assign(NewLNI,D+'Temp$.LNI');
    assign(OldLng,D+N+'.LNG');
    assign(NewLNG,D+'Temp$.LNG');
  end;

  GetFAttr(OldLNG,OldDAttr);
  GetFAttr(OldLNI,OldIAttr);
{$I-}
  erase(NewLNI);
  cnt:=ioresult;
  erase(NewLNG);
  cnt:=ioresult;


  reset(OldLNI);
  Seek(OldLNI,0);        {do color entries}
  read(OldLNI,index);

  rewrite(newLNI);
  seek(NewLNI,0);
  write(NewLNI,Index);

  seek(OldLNI,1);
  read(OldLNI,index);

  seek(NewLNI,1);
  write(NewLNI,Index);

  rewrite(NewLNG);
  close(NewLNG);
  cnt:=ioresult;



  reset(OldLNG,1);
  SizeOfData:=filesize(OldLng);
  TotalSize:=TotalSize+filesize(OldLNG);
  close(OldLNG);


  for cnt:=2 to filesize(OldLNI)-1 do begin
    seek(OldLNI,cnt);
    read(OldLNI,Index);

    if (Index.Size>0) and (index.Offset>=0) and (index.Size<10240000) and
      (index.Offset<SizeOfData) then begin
{      ((index.Offset+index.Size) < SizeOfData) }
      SizeTest:=index.offset;
      gotoxy(1,1);
      write(cnt,'  Copying ',index.size,' bytes from ',SizeTest,'.');
      clreol;

      if DSP then
        CopyData(Index.Size,Index.Offset,D+N+'.DSP',d+'temp$.DSP')
      else
        CopyData(Index.Size,Index.Offset,D+N+'.LNG',d+'temp$.LNG');
{      gotoxy(1,2);
      write('Offsets:  ',SizeTest,'=',index.Offset);
      if Sizetest<>index.Offset then write('    Different! ',SizeTest-Index.Offset);
}{
if cnt<50 then begin repeat
until keypressed;
write(readkey)
end}


      end
    else begin
      index.size:=0;
      index.offset:=0;
      end;
    seek(NewLNI,cnt);
    write(NewLNI,Index);
    end;

  reset(NewLNG,1);
  SizeNew:=filesize(newlng);
  Saved:=saved+SizeNew;
  close(NewLNG);
  close(OldLNI);
  erase(OldLNI);
	erase(OldLNG);
  close(NewLNI);

  if dsp then begin
	  rename(NewLNI,d+n+'.DSI');
	  rename(NewLNG,d+n+'.DSP');
  end else begin
	  rename(NewLNI,d+n+'.LNI');
	  rename(NewLNG,d+n+'.LNG');
  end;
  SetFAttr(NewLNI,OldIAttr);
  SetFAttr(NewLNG,OldDAttr);

  Dostatus;

  gotoxy(1,19);
  writeln(N,'.LNG: Original size: ',SizeOfData,'  New size: ',sizenew,
    '  Savings: ',SizeOfData-sizenew);
  close(NewLNG);

end;

procedure DoPacking;
var  counter,
     pfft:longint;
     pfft2:word;
begin
  if PackDesc then begin
    {Pack descriptions}
    DoStatus;
    FindFirst(dir+'*.DIR', AnyFile, DirInfo);
    repeat
       fsplit(Dirinfo.name,D,Name,Ext);
       assign(OldDesc,dir+name+'.TMP');
       erase(olddesc);

       assign(OldDesc,dir+Name+'.des');
       getfattr(OldDesc,OldAttr);
       rename(OldDesc,dir+name+'.TMP');

       reset(olddesc);
       TotalSize:=TotalSize+filesize(OldDesc);
       AreaTotal:=Filesize(OldDesc);
       AreaSaved:=0;
       close(olddesc);

       assign(NewDesc,dir+Name+'.des');
       rewrite(NewDesc);
       close(Newdesc);

       assign(Unew,dir+name+'.dir');
       reset(Unew);
       numfiles:=filesize(Unew)-1;

       DoStatus;

       for counter:=0 to filesize(unew)-1 do begin
         pfft:=0;
         pfft2:=1;
         ReadMsgData(pfft,pfft2,data,dir+name+'.des');


         seek(Unew,counter);
         read(Unew,NewU);
         gotoxy(1,1);
         write(counter,'  Reindexing ',newu.filename);
         clreol;
         CopyData(NewU.DescSize,NewU.DescOffset,dir+name+'.tmp',dir+name+'.des');
         seek(uNew,counter);
         write(uNew,NewU);
         end;      {0 to numfiles}


       close(unew);

       assign(NewDesc,dir+Name+'.des');
       reset(NewDesc);
       Saved:=Saved+filesize(NewDesc);
       AreaSaved:=filesize(newdesc);
       close(NewDesc);

       erase(OldDesc);

       Gotoxy(1,19);
       writeln('Area '+dir+name+': ',Areatotal,' initially, ',AreaSaved,' now.  ',Areatotal-AreaSaved,' Saved.');
       DoStatus;
       FindNext(DirInfo);
    until (doserror<>0);
  end;



  if PackMail then
    DoPackBase(dir+'mail.idx',DelDate);

  if PackBase then begin
    {Pack descriptions}
    DoStatus;
    FindFirst(dir+'*.IDX', AnyFile, DirInfo);
    repeat
       DoPackBase(dir+dirinfo.name,DelDate);
       DoStatus;
       FindNext(DirInfo);
    until (doserror<>0);
    end;

  if PackLang then begin
    {pack language files}
    DoStatus;
    findfirst(dir+'*.LNI',anyfile,dirinfo);
    repeat
       DoStatus;
       DoPackLang(dir+dirinfo.name,false);
       FindNext(DirInfo);
    until (doserror<>0);
    findfirst(dir+'*.DSI',anyfile,dirinfo);
    repeat
       DoStatus;
       DoPackLang(dir+dirinfo.name,true);
       FindNext(DirInfo);
    until (doserror<>0);
    end;
end;


begin
  PackDesc:=false;
  PackMail:=false;
  PackBase:=false;
  PackLang:=false;
  DelDate:=0;
  if paramcount>0 then begin
    writeln('CELPACK v1.2                                    A Celerity Utility');
    getdir(0,dir);
    if (paramcount>1) then begin
      moo:=paramstr(2);
      if ((moo[1]='-') or (moo[1]='/')) and (upcase(moo[2])='T') then
        val(copy(moo,3,length(moo)-2),DelDate,numfiles)
      else
        dir:=Moo;
      if paramcount>2 then begin
        moo:=paramstr(3);
        if ((moo[1]='-') or (moo[1]='/')) and (upcase(moo[2])='T') then
          val(copy(moo,3,length(moo)-2),DelDate,numfiles);
        end;
      end;

    moo:=paramstr(1);
    case upcase(moo[2]) of
      'B':PackBase:=true;
      'F':PackDesc:=true;
      'E':PackMail:=true;
      'L':PackLang:=true;
      else begin
          writeln;
          writeln('CELPACK is a Celerity v2.0 utility designed to pack various Celerity data');
          writeln('files into more efficient, smaller files by removing old, deleted, or');
          writeln('no longer used information.');
          writeln;
          writeln('SYNTAX:  CELPACK -option [Directory] [-Txx]');
          writeln;
          writeln('Valid Options:  -B   Pack Message Bases');
          writeln('                -F   Pack File Area Descriptions');
          writeln('                -E   Pack Electronic Mail');
          writeln('                -C   Pack CelerityText (Language/Menus) Files');
          writeln;
          writeln('If you do not specify a directory, the current directory will be used.');
          writeln;
          writeln('An optional date modifier may be used with the -E option.  The date');
          writeln('modifier can be used to delete electronic mail posted over xx days ago.');
          writeln('This is useful to clean up old mail which users have saved or not read.');
          writeln('It is recommended to periodically delete mail older than 90 or 180 days.');
          writeln;
          writeln('For your safety, please make periodical backups of your BBS data files.');
          halt(0);
          end;
      end;
  end else begin
    clrscr;
    writeln('CELPACK v1.2                                    A Celerity Utility');
    writeln;
    writeln('To improve performance of a Celerity v2.0 system, it is recommended that');
    writeln('the data files be packed from time to time.  This utility will pack all');
    writeln('files in the following categories:');
    writeln('*.DIR / *.DES        :  File area descriptions');
    writeln('MAIL.IDX / MAIL.DAT  :  Electronic mail system');
    writeln('*.LN? / *.DS?        :  CelerityText (Language/Menus) files');
    writeln('*.IDX / *.DAT        :  Message bases');
    writeln;
    writeln('This utility may be used in a command-line mode.  Type CELPACK /? for more.');
    writeln;
    writeln;
    writeln('Hit ^Break if you have not made a backup.');
    writeln;

    writeln('[1]  Pack File Descriptions  (*.DIR / *.DES)');
    writeln('[2]  Pack CelerityText Files (*.LN? / *.DS?)');
    writeln('[3]  Pack Electronic Mail    (MAIL.IDX / MAIL.DAT)');
    writeln('[4]  Pack Message Bases      (*.IDX / *.DAT)');
    write('(>');
    readln(moo);
    case upcase(moo[1]) of
      '1':PackDesc:=true;
      '2':PackLang:=true;
      '3':begin
          PackMail:=true;
          writeln('Delete mail older than how many days (0 keeps all mail)?');
          write('(>');
          readln(moo);
          val(moo,DelDate,numfiles);
          end;
      '4':PackBase:=true;
      end;

    writeln;
    writeln('Please enter the full pathname of your the directory to pack (ie: the mail');
    writeln('directory for email, conference directory for message bases, data directory');
    writeln('for file areas, language directory for language files):');
    getdir(0,dir);
    writeln('Or press ENTER for : ',dir);
    write('(>');
    readln(moo);
    if length(moo)>5 then dir:=moo;
    end;
  if dir[length(dir)]<>'\' then dir:=dir+'\';
  chdir(dir);
  TotalSize:=0;
  Saved:=0;
  clrscr;

  DoPacking;


  end.


'1/25/93 4/19/93 6/14/93 7/14/93 9/16/93 by Brendon Woirhaye');


9/16/93: Changed record format to support newer 2.0 messages/mail
