{$O-}
uses dos,crt;

{**************************************************************************}
{                                                                          }
{  AREAREC Record                                                          }
{                                                                          }
{  The AreaRec record format defines the structure of all *.XFR conference }
{  items (transfer areas).  It is a typed file containing a list of all    }
{  file areas in the transfer conference.                                  }
{                                                                          }
{**************************************************************************}
type arearec=record               {*.XFR Transfer Area record for v2.00 }
     name,                        { Name of xfer volume}
     xmodemdir:string[80];        { Pathname to files in section}
     AccessCode:string[50];       { Access string }
     unu1,
     unu2,
     unu3,
     unu4,
     unu5:byte;                   { Currently unused }
     AskAboutDiz,                 { Allow user to override File_id.DIZ }
     ImportDiz,                   { Import file_id.diz/desc.sdi if it exists }
     ScanAllAreas,                { Scan other areas when uploading }
     ScanThisArea,                { Scan this area in upload scan }
     FreeArea,                    { Free download area }
     NoRatio,                     { Ignore ratio }
     slowdrive:boolean;           { If "Slow drive", like a WORM, the system
                                    wont check for existance of file when
                                    listing files.  Additionally, files will
                                    be copied to the temporary directory
                                    before a download is made }
     unused:integer;              { }
     upload,                      { Allow uploads to section}
     download:boolean;            { Allow downloads from section}
     dataname:string[80];         { Path to the .DIR/.DES data files}
   end;

{**************************************************************************}
{                                                                          }
{  MSGDATAHEADER Record                                                    }
{                                                                          }
{  The MsgDataHeader is a very important record, but is never used in of   }
{  itself.  For all CelerityText data entries (messages, file descriptions,}
{  menus, language file entries, email, etc), there is an index file and   }
{  a data file.  The index file is always a typed file (see other sections }
{  for details on these record formats), but the data file is dynamic.     }
{  This is where the MsgDataHeader comes in.  At the very beginning of a   }
{  data segment in a CelerityText data file is a special 9-byte header.    }
{  This header is the MsgDataHeader, which contains some important details }
{  about the data that follows.  The IndexOffset refers back to the index  }
{  file and contains the index number of the record which is associated    }
{  with the data segment.  The size tells Celerity how many bytes are in   }
{  the data segment, and CompType refers to the "compression format",      }
{  which is currently unused.                                              }
{                                                                          }
{**************************************************************************}
type	MsgDataHeader = Record
		  IndexOffset	: longint;	{Location of index entry}
		  Size				: longint;	{Size of message}
		  CompType		: byte;			{Compression type - unused in 2.0}
		  end;


type
    DataPtr = ^byte;

{**************************************************************************}
{                                                                          }
{  MSGINDEXREC Record                                                      }
{                                                                          }
{  The MsgIndexRecord is vital to the Celerity v2.00+ messaging system.    }
{  The *.IDX index files in the message bases and MAIL.IDX file are typed  }
{  files based on this record.  It contains all pertinent information      }
{  regarding messages on a Celerity system.                                }
{                                                                          }
{**************************************************************************}
type MsgIndexRec = record     {*.IDX format for Celerity v2.00}
		 Title			:	string[71];	{Title of the message}
		 Author		:	string[35];   {Handle of message author}
		 AuthorAddr:string[15];	  {This is the address of the author}
     AuthorNote: string[30];  {Authors User Note}
		 LeftTo		: string[35];   {Handle of addressee}
		 LeftAddr	:	string[15];	  {Address of the recipient}
		 ForwardBy	: string[35]; {Handle of the user who forwarded the message}
		 ForwardAddr:string[15];	{This is the address of the fowarder}
		 When			: longint;		  {When message was written in PACKTIME}

		 TimesRead	: word;				{Message read how many times??}
		 Return		: boolean;		  {Return Receipt requested}
		 Anon			: boolean;		  {Anonymous message}
		 FileName	: string[12];   {Name of attached file}

		 Offset		: longint;		  {position of data in data file}
		 Size			: longint;		  {size of message in bytes}

		 {flags....}
		 Deleted		: boolean;		{message is deleted}
     Validated : boolean;     {message validatation state}

		 {Thread Info}
		 Origin,             		  {Original post of thread}
		 Parent,									{Parent of this message}
     PrevSibling,             {Prev reply to the original message. (Sibling)}
		 NextSibling,						  {Next reply to the original message (sibling)}
		 Child			: longint;		{First reply to THIS post.. eldest "child"}

     MsgRefId  : longint;     {Reference to this post}
     FidoToss,
     Toss2,
     Toss3,                   {Flags for third-party utilities}
     ExtTossed : boolean;     {External utility tossed}
     Space1,                  {Spare variable space....}
     Space2,
     Space3,
     Space4 : longint;
		 end;


{**************************************************************************}
{                                                                          }
{  FILEREC Record                                                          }
{                                                                          }
{  The FileRec record (UdRec in 1.42 and previously) is the record used for}
{  all file listings in Celerity v2.00+ transfer sections.  All Celerity   }
{  *.DIR files are made up of this record, and have a corresponding .DES   }
{  data file (CelerityText with MsgDataHeader - see above) for description }
{  data.  The DescOffset and DescSize refer to this .DES file.             }
{                                                                          }
{**************************************************************************}
type FileRec=record                 { *.DIR for version 2.00+ 7/14/93}
       sentby:string[30];           { Who uploaded the file}
       when,                        { When was it uploaded - PACKTIME}
       whenrated:longint;           { Show up on newscans since.. PACKTIME}
       filename,                    { DOS filename}
       dlpw:string[15];             { Password to download it}
       path:string[50];             { Full pathname pointing to location}
       points:integer;              { Cost to download}
       filesize:longint;            { Size (in bytes)}
       SystemNote:string[44];       { Contains system messages re:viruses etc}
       DescOffset,                  { Offset of description data}
       DescSize,                    { Size of description}
       ExtDescOffset,               { Offset of extended description}
       ExtDescSize:longint;         { Size of extended description}
       DescLength:byte;             { Length (# of lines) of description}
       DateMoved:longint;           { Date the file was last moved}
       Unused:string[14];           { unused with 2.0}
       downloaded:integer;          { Number of times file downloaded}
       newfile,                     { Unvalidated file}
       FreeFile:boolean;            { File is a free download}
       progname:string[20];         { This is the full/program name of the program}
       ofwhat1,ofwhat2:byte;        { These are for "disk x of y"}
       ptsgiven:integer;            { How many validation points awarded}
       private:string[30];          { Who is a private file for}
     end;


     OldUdrec=record        {Old 1.23 / 1.42 file record}
       sentby:string[30];
       when,whenrated:longint;
       filename,dlpw:string[15];
       path:string[50];
       points:integer;
       filesize:longint;
       descrip:string[80];
       downloaded:integer;
       newfile,specialfile:boolean;
       progname:string[20];         { This is the full name of the program}
       ofwhat1,ofwhat2:byte;        { These are for "disk x of y"}
       ptsgiven:integer;            { How many validation points awarded}
       private:string[30];                { Who is a private file for}
     end;

     {This dumps the extended description and shortens the download password}


     Oldarearec=record                      {Transfer Area}
       name,xmodemdir:string[80];
       sponsor,areapw:string[30];
       slowdrive:boolean;
       level:integer;
       upload,download:boolean;
       dataname:string[80];
     end;



  var unew:file of FileRec;
      uold:file of Oldudrec;
      newu:FileRec;
      oldu:OldUdRec;

      cnt,cnt2,conf,areanum,n,numfiles:integer;
      filename:string[80];
      afile:file of oldarearec;
      nuafile:file of AreaRec;
      area:oldarearec;
      nuarea:AreaRec;

 const
	{Constants for File Locking}
                        {The +2 allows opening for Read/Write}
	RW				=	 0+2;	    {Fully Open}
	Locked		= 16+2;	    {Read/Write Denied to other programs}
	ReadOnly	=	32+2;	    {Write Access Denied}
	WriteOnly	=	48+2;	    {Read Access Denied}

	Denied		=	 5;	      {File Locked Error}



function NetReset(var af; size:word; mode:byte):byte;
var
	cnt,
	initmode: byte;
	result	: word;
	f				: file absolute af;
begin
	initmode:=filemode;	{store the original filemode}

	filemode:=mode;			{Set the desired mode}

	if size=0 then		{Branch for Record Size}
		reset(f)
	else
		reset(f,size);

	result:=ioresult;

	cnt:=0;

	while (result=Denied) and (cnt < 200) do begin	{Approx 2 Seconds...}
		if size=0 then		{Branch for Record Size}
			reset(f)
		else
			reset(f,size);

		result:=ioresult;
		inc(cnt);					{Inc Loop Counter}
		delay(10);				{Delay 1/100 sec }

		end;

	FileMode:=InitMode;		{Restore original FileMode}

	NetReset:=Result;			{Return the result code}

end;


procedure SaveMsgData(var Index:MsgIndexRec; IndexLoc: longint; Data:dataPtr; FN: pathstr);
{ Save the actual message data to the end of the message data file }
{ IndexLoc needs to be passed in so that the data header can refer back}
var
	datafile	: file;
	offset		: longint;
	header		: MsgDataHeader;
	Result		: byte;

begin
	assign(datafile,FN);
	Result:=NetReset(datafile,1,Locked);

	case result of

		0:begin
			offset:=filesize(datafile);		{Get Filesize}
			seek(datafile,offset);				{Seek to EOF}

			Index.offset:=offset;         {Pos of the Data in data file}

			With Header do begin
				IndexOffset:=IndexLoc;    {This is the Loc of the INDEX}
				Size:=Index.Size;         {Above needs to be RefNumId}
				CompType:=0;
				end;

			blockwrite(datafile,header,sizeof(header) );

			blockwrite(datafile,data^,index.size);
			end;
		5:writeln('File Busy: ',FN);							{File Locked - Repeat??}

		else begin                                {Create the file first}
			Result:=NetReset(datafile,1,Locked);
			rewrite(Datafile,1);
      close(DataFile);

			Result:=NetReset(datafile,1,Locked);

			offset:=filesize(datafile);		{Get Filesize}
			seek(datafile,offset);				{Seek to EOF}

			Index.offset:=offset;         {Pos of the Data in data file}

			With Header do begin
				IndexOffset:=IndexLoc;      {This is the Loc of the INDEX}
				Size:=Index.Size;
				CompType:=0;
				end;

			blockwrite(datafile,header,sizeof(header) );

			blockwrite(datafile,data^,index.size);

			end;
	end; {case}

	close(datafile);
end;

function strr (i:integer):string;
var b:string[30];
begin
  str (i,b);
  strr:=b
end;

procedure GetFileDescript(var ud:FileRec;s:string);
var       Dummy:msgindexrec;
          IndexLoc:Longint;
          Data,TempData:dataPtr;
          cnt:byte;
          SizeofDesc:word;

begin
  if length(s)>48 then s:=copy(s,1,48);
  fillchar(ud.SystemNote,sizeof(ud.SystemNote),0);
  fillchar(ud.Unused,sizeof(ud.Unused),0);

  SizeofDesc:=0;
  ud.DescLength:=0;
  GetMem(Data,8192);
  TempData:=data;
      for cnt:=1 to length(s) do begin
        TempData^:=ord(s[cnt]);
        inc(Tempdata);
        inc(SizeOfDesc);
        end;
  inc(ud.DescLength);
  Dummy.size:=SizeOfDesc;
  SaveMsgData(Dummy,IndexLoc,Data,area.DataName+'.DES');
  ud.DescSize:=Dummy.Size;
  freemem(data,8192);
  ud.DescOffset:=Dummy.offset;
end;


procedure convert;
 var m,o:integer;
 begin
    with newu do begin
      sentby:=oldu.sentby;
      when:=oldu.when;
      whenrated:=oldu.whenrated;
      filename:=oldu.filename;
      path:=oldu.path;
      points:=oldu.points;
      filesize:=oldu.filesize;
      SystemNote:='';
      dlpw:=oldu.dlpw;
      downloaded:=oldu.downloaded;
      newfile:=oldu.newfile;
      FreeFile:=false;
      progname:=oldu.progname;
      ofwhat1:=oldu.ofwhat1;
      ofwhat2:=oldu.ofwhat2;
      ptsgiven:=oldu.ptsgiven;
      private:=oldu.private;
      fillchar(Unused,sizeof(unused),#0);
      DescOffset:=0;
      DescSize:=0;
      ExtDescOffset:=0;
      ExtDescSize:=0;
    end;
    GetFileDescript(NewU,OldU.Descrip);
 end;


var dir,moo:string[80];
var
 DirInfo: SearchRec;         { For Windows, use TSearchRec }
 P,Name,E:string;

  begin
    writeln('This program will convert all Celerity 1.23/1.42 .DIR files to the new 2.0');
    writeln('.DIR/.DES format supporting multiple line descriptions.');
    writeln;
    writeln('6/5/90, 7/5/90, 1/18/93, 6/11/93, 6/23/93, 7/14/93 by Brendon Woirhaye');
    writeln;
    writeln('Hit ^Break if you have not made a backup.');
    writeln;
    writeln('Please enter the FULL path of your data directory.  Include drive spec.');
    getdir(0,dir);
    writeln('Or press ENTER for : ',dir);
    write('-->');
    readln(moo);
    if length(moo)>5 then dir:=moo;
    if dir[length(dir)]<>'\' then dir:=dir+'\';
    exec(getenv('COMSPEC')+'cd ','cd '+dir);

    {Seek all the areas in conferences}
    FindFirst('*.DIR', AnyFile, DirInfo);
    while DosError = 0 do
    begin
      Writeln('Converting ',DirInfo.Name);
      assign(uold,Dirinfo.Name);
      assign(unew,'temp');
      rewrite(unew);
      reset(uold);
      numfiles:=filesize(uold)-1;
      writeln('      Converting ',numfiles,' file records..');
      write('        File #');
      fsplit(dirinfo.name,P,Name,E);
      area.dataname:=p+name;

      for cnt:=0 to numfiles do begin
        write(cnt);
        for cnt2:=1 to length(strr(cnt)) do write(#8);
        seek(uold,cnt);
        read(uold,oldu);
        seek(unew,cnt);
        convert;
        write (unew,newu);
      end;      {0 to numfiles}

      writeln;
      close(unew);
      close(uold);
      erase(uold);
      rename(unew,Dirinfo.Name);
      writeln('      Area complete.');
      FindNext(DirInfo);
    end;
  end.


{
 This program is NOT a stunning example of good program code.  It has been
 hacked through about half a dozen programs over three years or so.  Still,
 it is functional and quick.
 }
