program Log2Text;

uses Dos,SysLog;

{**************************************************************************}
{                                                                          }
{  LOGREC Record                                                           }
{                                                                          }
{  The LogRec record contains the format for the system logs, which reside }
{  in the DATA directory and are named SYSLOG.x (x being the node number   }
{  of the log).  The log entries are referenced with a two-dimensional     }
{  numbering system - Menu and Subcommand.  These values are listed below. }
{                                                                          }
{**************************************************************************}
type logrec=record                 {SYSLOG.x format for 2.00+}
       menu,subcommand:integer;    {Identifier - see below}
       param:string[41];           {Parameter passed}
       when:longint                {PACKTIME format of entry time}
     end;

function strr (i:integer):string;
var b:string;
begin
  str (i,b);
  strr:=b
end;

function timestr (time:longint):string;
var h1:integer;
    ms,hs:string;
    dt:datetime;
const ampmstr:array [false..true] of string[2]=('am','pm');
begin
  unpacktime (time,dt);
  h1:=dt.hour;
  if h1=0
    then h1:=12
    else if h1>12
      then h1:=h1-12;
  ms:=strr(dt.min);
  if dt.min<10 then ms:='0'+ms;
  hs:=strr(h1);
  if length(hs)=1 then
    hs:='0'+hs;
  timestr:=hs+':'+ms+' '+ampmstr[dt.hour>11]
end;

function datestr (time:longint):string;
var dt:datetime;

	function PadZero(s:string):string;
	begin
		if length(s)=1 then
			s:='0'+s;
		PadZero:=s;
		end;
begin
  unpacktime (time,dt);
  datestr:=padzero(strr(dt.month))+'/'+padzero(strr(dt.day))+'/'+strr(dt.year-1900)
end;


var Log:file of logrec;
    Entry:logrec;
    f:text;
    AppendFlag:boolean;
    DeleteFlag:boolean;
    LogPath:string;
    TextPath:string;
    LogString:string;
    i,n:integer;
    Done:boolean;
    Finished:boolean;


  procedure ShowHelp;
  begin

    writeln('Syntax:  LOG2TEXT.EXE <path to log> <path to text> [/a /d]');
    writeln;
    writeln('/a = Append to destination (text) file');
    writeln('/d = Delete source (log) file');
    writeln;
    writeln('LOG2TEXT.EXE is a simple utility for Celerity v2.00 which will');
    writeln('take a Celerity BBS log file and convert it to ASCII text for easy');
    writeln('viewing, archiving, and manipulation.');
    writeln;
  end;



  function firstentry(l:logrec):boolean;
  begin
    firstentry:=(l.menu=0) and (l.subcommand in [0])
  end;


begin
  writeln('LOG2TEXT.EXE v1.0');
  writeln;
  if paramcount<2 then begin     {If no parameters, show help and exit}
    ShowHelp;
    exit;
    end;

  AppendFlag:=false;
  for i:=1 to paramcount do
    if (paramstr(i)='/a') or (paramstr(i)='/A') then
      AppendFlag:=true;

  DeleteFlag:=false;
  for i:=1 to paramcount do
    if (paramstr(i)='/d') or (paramstr(i)='/D') then
      DeleteFlag:=true;


  LogPath:=paramstr(1);
  TextPath:=paramstr(2);


  {$I-}
  assign(log,LogPath);
  reset(log);
  i:=ioresult;
  if i<>0 then begin
    close(log);
    writeln('Error #',I,' reading ',logpath);
    exit;
    end;

  assign(f,textpath);
  if AppendFlag then begin
    Append(f);
    i:=ioresult;
    if i<>0 then begin
      writeln('Error #',i,' appending ',textpath);
      if i<>2 then exit;
      rewrite(f);
      end;
    end
  else
    rewrite(f);


  n:=0;
  finished:=false;
  repeat
{


    done:=false;
    while (n<filesize() and not done do begin
      dec(n);
      seek (log,n);
      read (log,entry);
      if firstentry(entry) then done:=true;
      end;
}

    if n=filesize(log) then
      Finished:=true;

    seek (Log,n);
    read (Log,entry);
    if firstentry(entry) then begin
      writeln(f);
      writeln;
      end;

    LogString:=SysLogText(entry.menu,entry.subcommand);

    i:=pos('%',LogString);
    if i<>0 then
      Logstring:=copy(LogString,1,i-1)+entry.param+copy(LogString,i+1,255);
    writeln(f,datestr(entry.when),' ',timestr(entry.when),'  ',LogString);
    writeln(datestr(entry.when),' ',timestr(entry.when),'  ',LogString);
    inc(n);
  until finished;

  close(log);
  close(f);
  if DeleteFlag then
    erase(log);
  writeln('Complete!');

end.