/**************************************************************************}
{                                                                          }
{  USERREC Record                                                          }
{                                                                          }
{  The USERREC record is the format for the USERS file found in the        }
{  Celerity main BBS directory.  It contains the bulk of the information   }
{  about the user's personal details, user configuration, BBS statistics,  }
{  and so forth.  The USERS file is a typed file of USERREC records.       }
{                                                                          }
{  Note: Record #0 of the USERS file should ALWAYS be blank except for the }
{        REALNAME field, which contains the version number of the user     }
{        list in the form "2.00" or "1.42".                                }
{                                                                          }
{**************************************************************************/

typedef struct                                  /* Version 2.00 */
 {
  long                 UserID;                  /* User identification number */
  char                 Handle[31],              /* Users handle */
		       Realname[31],            /* Users real first/last name */
		       InterNetName[16],        /* name for internet mail */
		       Signature[81],           /* users signature */
		       Address[2][31],          /* Street address */
		       Macro[3][31],            /* macros */
		       Note[31],                /* user note */
		       PrivNote[81],            /* sysop private note */
		       Password[16],            /* password */
		       Phonenum[13],            /* Phone # */
		       DataPhone[13],           /* Data Phone # */
		       FaxPhone[13];            /* Fax phone # */
  long                 Laston;                  /* Last time on in PACKTIME */
  int                  NumCalls,                /* nuumber of calls */
		       Udlevel,                 /* Xfer level */
		       Level,                   /* main bbs level */
		       HelpLevel;               /* Novice / Expert Mode */
  long                 udpoints,                /* file pts. */
		       AccessFlags,             /* Access Flags 1 - 32 */
		       XferConfAccess;          /* unused */
  unsigned char        AutoChatCall;            /* Call chat upon login */
  int                  infoforms[5];            /* Location values for infoforms */
  long                 InfoScrOffset[10],       /* unused */
		       InfoScrSize[10];         /* unused */
  unsigned char        Color[8],                /* User color values. Setting to 0 will cause the
						   system to remap them */
		       displaylen;              /* users display length 10 - 70 lines */
  int                  lastmessages,            /* how many new posts? */
		       lastups;                 /* new uploads */
  unsigned char        SingleKey,               /* use single key input */
		       defproto,                /* Default xfer prot */
		       hack;                    /* hack attempts */
  long                 AccountCreated,          /* Data account was create in PACKTIME */
		       Dummy1[20];              /* usused */
  unsigned char        Dummy2,                  /* usused */
		       TimeDoor,                /* For use with Timedoor */
		       LangSelect,              /* language file, if 0, will ask */
		       MenuSelect;              /* menu file, if 0, will ask */
  long                 SpareLongs[17];          /* unused */
  unsigned int         lastcps,                 /* average cps rate */
		       cpstimes;                /* times counted for average cps rate */
  unsigned char        CurList;                 /* which file list config in use? */
  int                  commpoints,              /* commision pts since last call */
		       valpoints;               /* validation pts since last call */
  unsigned char        ScanNet,                 /* scan netted subs? */
		       userpcr,                 /* user's PCR */
		       userudr,                 /* user's UDR */
		       usergenr,                /* user's GENR */
		       age,                     /* user's Age */
		       usersex;                 /* user's sex. TRUE = female, False = male */
  long                 Expiration,              /* expiration date in PACKTIME */
		       birthdate;               /* birthdate in PACKTIME */
  unsigned char        voteyes,                 /* number of yes votes */
		       voteno,                  /* number of no votes */
		       timeperday;              /* time per day */
  int                  Posts_Month,             /* # of posts for the month */
		       Posts_all,               /* Total # of posts */
		       Uploads_Month,           /* # of uploads for the month */
		       Uploads_All,             /* Total # of uploads */
		       Dnloads_Month,           /* # of downloads for the month */
		       Dnloads_all;             /* Total # of downloads */
  long                 UpBytes_Month,           /* # of uploaded bytes for the month */
		       UpBytes_All,             /* Total # of uploaded bytes */
		       DnBytes_Month,           /* # of downloaded bytes for the month */
		       DnBytes_All,             /* Total # of downloaded bytes */
		       PBytes_All,              /* Total # of posted bytes */
		       PBytes_Month;            /* # of posted bytes for the month */
  unsigned int         KperDay,                 /* Total # of K allowed per day for download */
		       KToday;                  /* # of K downloaded today */
  unsigned char        NoNet,                   /* No net access */
		       Exempt[5],               /* Exemptions 1-pcr, 2-udr, 3-genr, 4-???, 5-xfer */
		       SpecialAccess,           /* Unused */
		       Naplps,                  /* Use Naplps */
		       RIP,                     /* Use Rip */
		       QwkPacker,               /* Code for QWK packer */
		       UseYesNo,                /* Use Yes/No prompts */
		       UseFloat,                /* Use floating prompts */
		       UseColor,                /* Use Color */
		       UseSignature,            /* Use Signature */
		       UseFSE,                  /* Use Full Screen Editor */
		       PermitPaging,            /* Allow paging between nodes */
		       SkipTopTen,              /* View top ten? */
		       SavedTermEmul;           /* 0 = none, 1 = vt52, 2 = v5t Atari, 3 = vt100, 4 = ANSI, 5 = Avatar */
  long                 SysopFlags,              /* Sysop flags #1 - 32 */
		       AreaAcess;               /* unused */
  unsigned char        VoteData[8];             /* vote data */
  long                 RemainingTimeCredit,     /* time credit remaining for today */
		       TermCharacter,           /* bitfield: 1 = uselinefeeds,
						   2 = post prompts, 3 = more prompts,
						   4 = 8-bit ascii */
		       FileList[3],             /* Filelisting config. Bitfield :
						   1 = name, 2 = extention, 3 = progname
						   4 = disks, 5 = size, 6 = points, 7 = uploader,
						   8 = date u/l, 9 = times d/l, 10 = description,
						   11 = skipline */
		       TotalTimeOnline;         /* Total Time Online */
 } UserRec;

/**************************************************************************}
{                                                                          }
{  BBSREC Record                                                           }
{                                                                          }
{  The BBSREC record is used to hold data for the BBS list feature of      }
{  Celerity.  BBS Lists are held in *.LST files found in the conference    }
{  directory.  The main BBS list is BBSLIST.LST, others may be created     }
{  within the conference structure.  It is a typed file.  If the file is   }
{  deleted, it will be re-created with the official Celerity support       }
{  boards listed.                                                          }
{                                                                          }
{**************************************************************************/

typedef struct                      /* BBSLIST.LST for Celerity v2.0 */
 {
  char              name[25],       /* Name of BBS */
		    Descript[61],   /* Description of BBS */
		    Number[21];     /* BBS Phone Number */
  unsigned char     DeviceType;     /* 0=BBS, 1=VMB, 2=NUA */
  char              software[9];    /* Software Type */
  unsigned int      megs;           /* Megabytes of storage */
  char              maxbaud[6];     /* Maximum BPS rate in ASCII form */
  unsigned char     lines,          /* # of incoming phone lines */
		    mnp,            /* Supports MNP error correction */
		    ZYX,            /* Supports Zyxel 19.2 connection */
		    PEP,            /* Supports Telebit protocol */
		    HST,            /* Supports USRobotics HST standard */
		    V32,            /* Supports ITU-T v.32 standard */
		    V42,            /* Supports ITU-T v.42 error correction */
		    V42bis,         /* Supports ITU-T v.42bis compression */
		    V32bis,         /* Supports ITU-T v.32bis standard */
		    V32ter,         /* Supports V.32Ter interrim standard */
		    VFC,            /* Supports V.FC interrim standard */
		    V34,            /* Supports ITU-T V.34 (v.fast) */
		    ISDN,           /* Supports ISDN connection */
		    Allow56k,       /* Supports 56k ISDN connection */
		    Allow64k,       /* Supports 64k ISDN connection */
		    Allow128k,      /* Supports 128k ISDN multiplexing */
		    Allow144k,      /* Supports 144k ISDN multiplexing */
		    Allow1,         /* unused */
		    Allow2,         /* unused */
		    Allow3,         /* unused */
		    Allow4,         /* unused */
		    Allow5,         /* unused */
		    Allow6,         /* unused */
		    Allow7,         /* unused */
		    Allow8,         /* unused */
		    Allow28800,     /* Supports 28,800bps connection */
		    Allow26600,     /* Supports 26,600bps connection */
		    Allow24000,     /* Supports 24,000bps connection */
		    Allow21600,     /* Supports 21,600bps connection */
		    Allow19200,     /* Supports 19,200bps connection */
		    Allow16800,     /* Supports 16,800bps connection */
		    Allow14400,     /* Supports 14,400bps connection */
		    Allow9600,      /* Supports 9600bps connection */
		    Allow2400,      /* Supports 2400bps connection */
		    Allow1200;      /* Supports 1200bps connection */
  char              leftby[31];     /* Handle of who ever made this entry */
  long              when;           /* When it was made in PACKTIME */
 } BBSrec;

/**************************************************************************}
{                                                                          }
{  TOPTENREC Record                                                        }
{                                                                          }
{  The TOPTENREC record contains the data for Celerity's Top Ten Users     }
{  feature.  The Top Ten data is held in the TOPTEN.DAT file in the DATA   }
{  directory.  It is a typed file, but has a unique structure.  Record #0  }
{  is unused, and can be used for third party development if desired.      }
{  Records #1 through #10 are used to store the top ten all time posters.  }
{  Records #11 through #20 are used for the top ten monthly posters.       }
{  Records #21 through #30 are for the top kbyte uploaders of all time.    }
{  Records #31 through #40 are for the top monthly kbyte uploaders.        }
{  Records #41 through #50 are for the top kbyte downloaders of all time.  }
{  Records #51 through #60 are for the top monthly kbyte downloaders.      }
{                                                                          }
{  Records above #60 are currently unused by Celerity, but are defined for }
{  third-party utility use and future expansion.  They are:                }
{                                                                          }
{  Records #61 through #70 are for the top downloaded files of all time.   }
{  Records #71 through #80 are for the top downloaded files this month.    }
{  Records #81 through #90 are for the top ten callers of all time.        }
{  Records #91 through #100 are for the top ten callers of the month.      }
{                                                                          }
{  The top ten file works through usage, and will eventually recreate      }
{  itself if it becomes damaged or deleted.  When a user logs onto the     }
{  board, enters the transfer section, or enters the message bases, the    }
{  top ten will be scanned for the user and updated.                       }
{                                                                          }
{**************************************************************************/

typedef struct                      /* TOPTEN.DAT for Celerity v2.0 */
 {
  char              name[31];       /* Name of User / file */
  long              kup,            /* Used for kbytes uploaded */
		    kdn;            /* kbytes downloaded */
  unsigned int      numup,          /* Used for number of uploads */
		    numdn,          /* downloads */
		    posts;          /* Used for the number of posts */
  unsigned char     code;           /* Used to determine which class it belongs to - */
 } TopTenRec;                       /* 1 - top posters, 2 monthly posters, etc */

/**************************************************************************}
{                                                                          }
{  NEWUSERVOTEREC Record                                                   }
{                                                                          }
{  The NewUserVoteRec contains information about users currently in the    }
{  new user voting process.  It is a typed file, and position in the file  }
{  does not matter.  When a new entry is added, the file is scanned for an }
{  entry with a blank "handle" field.  When an entry is deleted, the handle}
{  field is erased.  The new user data is stored in USERVOTE.DAT in the    }
{  DATA directory.                                                         }
{                                                                          }
{**************************************************************************/

typedef struct                         /* USERVOTE.DAT for version 2.0 */
 {
  char         handle[31];             /* Handle of the user being voted on */
  long         UserID,                 /* UserID of user */
	       voteyes[100],           /* Yes Votes */
	       voteno[100];            /* No Votes */
  char         Comment[20][85];        /* Comments about the user */
 } NewUserVoteRec;

/**************************************************************************}
{                                                                          }
{  USERSPECSREC Record                                                     }
{                                                                          }
{  The UserSpecsRec contains template information for the old user editor  }
{  and for the Zap User process.  Basically, it allows the sysop to define }
{  ranges to list/delete users by.  The file USERSPEC is in the DATA dir.  }
{                                                                          }
{  Please note that this section is due for an overhaul, and will probably }
{  be changed considerably in 2.10                                         }
{                                                                          }
{**************************************************************************/
/*
type userspecsrec=record   {USERSPEC for version 2.0}
		 name:string[30];
		 minlevel,maxlevel,minlaston,maxlaston:integer;
		 minpcr,maxpcr:real
		end;   */

/* NOTE : This record is currently not editable in C due to the 'real' types
   currently used, which are not compatible with C */

/**************************************************************************}
{                                                                          }
{  MSGREC Record                                                           }
{                                                                          }
{  The MSGREC record is no longer used externally by Celerity.  It remains }
{  here because it is used for the Full Screen / Line Editors for Celerity }
{  and for the quoting system.  Additionally, it is used for CelerityNet   }
{  for version 1.42.                                                       }
{                                                                          }
{**************************************************************************/

#define maxmessagesize 100

typedef struct                         /* Only used internally in 2.0 */
 {
  char              text[maxmessagesize][81],
		    leftto[31],
		    title[81];
  unsigned char     anon;
  int               numlines;
 } MsgRec;

/**************************************************************************}
{                                                                          }
{  MSGDATAHEADER Record                                                    }
{                                                                          }
{  The MsgDataHeader is a very important record, but is never used in of   }
{  itself.  For all CelerityText data entries (messages, file descriptions,}
{  menus, language file entries, email, etc), there is an index file and   }
{  a data file.  The index file is always a typed file (see other sections }
{  for details on these record formats), but the data file is dynamic.     }
{  This is where the MsgDataHeader comes in.  At the very beginning of a   }
{  data segment in a CelerityText data file is a special 9-byte header.    }
{  This header is the MsgDataHeader, which contains some important details }
{  about the data that follows.  The IndexOffset refers back to the index  }
{  file and contains the index number of the record which is associated    }
{  with the data segment.  The size tells Celerity how many bytes are in   }
{  the data segment, and CompType refers to the "compression format",      }
{  which is currently unused.                                              }
{                                                                          }
{**************************************************************************/

typedef struct
 {
  long              IndexOffset,       /* Location of index entry */
		    Size;              /* Size of message */
  unsigned char     CompType;          /* Compression Type - Unused */
 } MsgDataHeader;

/**************************************************************************}
{                                                                          }
{  MSGINDEXREC Record                                                      }
{                                                                          }
{  The MsgIndexRecord is vital to the Celerity v2.00+ messaging system.    }
{  The *.IDX index files in the message bases and MAIL.IDX file are typed  }
{  files based on this record.  It contains all pertinent information      }
{  regarding messages on a Celerity system.                                }
{                                                                          }
{**************************************************************************/

typedef struct                   /* *.IDX format for Celerity v2.00 */
 {
  char          Title[72],       /* Title of message */
		Author[36],      /* handle of author */
		AuthorAddr[128], /* the address of author */
		AuthorNote[31],  /* Authors user note */
		LeftTo[36],      /* Person left to */
		LeftAddr[128],   /* address of person left to */
		ForwardBy[36],   /* handle of the person who forwarded msg */
		ForwardAddr[128];/* address of the person who forwarded msg */
  long          When;            /* when message was posted */
  unsigned int  TimesRead;       /* times msg was read */
  unsigned char Return,          /* Return reciept? */
		Anon;            /* annoymous message? */
  char          FileName[13];    /* Name of attached file */
  long          Offset,          /* position of data in data file */
		Size;            /* Size of message */
  unsigned char Deleted,         /* Deleted? */
		Validated,       /* validated? */
		Personal;        /* Private? */
  long          Origin,          /* original post of thread */
		Parent,          /* parent message of thread */
		PrevSibling,     /* previous reply */
		NextSibling,     /* next reply */
		Child,           /* first reply */
		MsgRefId;        /* reference to this post */
  unsigned char FidoToss,        /* FidoNet */
		Toss2,           /* Fido */
		Toss3,           /* Fido */
		ExtTossed;       /* For fidonet */
  unsigned char NetStatus;       /* unused */
  char          AreaName[21],    /* internal name of network */
		NetIdent[31];    /* Netname */
  unsigned char AssortedCrap[60];/* unused */
 } MsgIndexRec;

/**************************************************************************}
{                                                                          }
{  MSGBASEREC Record                                                       }
{                                                                          }
{  The MsgBaseRec record contains the format for Celerity's message base   }
{  configuration files (*.BSE in the conference directory).  It contains   }
{  all needed information about the actual message base, its configuration,}
{  restrictions, and so forth.                                             }
{                                                                          }
{**************************************************************************/
     
typedef struct                         /* *.BSE format for Celerity v2.00 */
 {
  char             Name[31],           /* Name of sub board */
		   QwkName[16],        /* Code for QWK packer */
		   AccessCode[16],     /* Access Code */
		   AttachedPath[80],   /* Path for attached files */
		   DataPath[80],       /* Path for data files */
		   IndexPath[80],      /* path for index files */
		   QscanPath[80],      /* path for qscan data */
		   OriginLine[81];     /* originline for sub board */
  int              BoardMsgLimit,      /* Max # of messages */
		   CurrentMsgCount;    /* Current # of messages */
  unsigned char    NetType;            /* Type of Network */
  char             NetAddress[16],     /* Net Address */
		   AreaName[21],       /* MercuryNet name */
		   BaseNews[81];       /* path for news */
  unsigned char    RealNames,          /* force real names */
		   AllowAnsi,          /* allow ansi */
		   AllowNaplps,        /* allow naplps */
		   AllowAnony,         /* allow anonymous messages */
		   AllowFileAttach,    /* allow attaching of files */
		   AllowPrivateMsg,    /* allow private messages */
		   AllowUpload,        /* allow uploading of messages */
		   AllowDnload,        /* allow downloading of messages */
		   RequireValidate,    /* require validation of messages */
		   TimeStopped,        /* time stopped while in sub */
		   option1,            /* unused */
		   option2,            /* unused */
		   option3,            /* unused */
		   option4,            /* unused */
		   option5;            /* unused */
  long             CurMsgRefID,        /* Message reference number */
		   Index;              /* Index number */
  unsigned int     QwkConfNum;         /* for QWK packer */
  unsigned char    Compression;        /* 0 = uncompressed - default */
  int              PostSizeLimit,      /* 0 = no limit. */
		   FileCredit;         /* bytes per credit point. if negative,
					  points per post */
  unsigned char    fidoLocalAttribute, /* Unused always "ON" */
		   fidoDelSentMsg,     /* Del .MSG's after sending ON */
		   fidoDelTossedMsg,   /* Del .MSG's after importing ON */
		   fidoSetCrash,       /* Set Crash attr. OFF */
		   fidoSetPrivate,     /* Set Private attr. OFF */
		   fidoSetHold,        /* Set Hold attr. ON */
		   fidoSetAtt7,        /* unused */
		   fidoSetAtt8;        /* unused */
  char             NetIdent[31];       /* Name of network */
  int              CTNetNumber;        /* Relative net number used by CelToScn */
  char             NetPassword[11],    /* Password for Networks */
		   Space[103];         /* Unused */
} MsgBaseRec;

/**************************************************************************}
{                                                                          }
{  QSCANREC Record                                                         }
{                                                                          }
{  The QscanRec record is used for retaining newscan information about     }
{  Celerity 2.00+ message and file areas.  When a user enters a .BSE or    }
{  .XFR area from a newscan, this associated file is accessed and scanned. }
{  If the user's UserID does not exist in the file, an entry will be made  }
{  for the user and he or she will be asked if he would like to always     }
{  scan this base, be asked every time, or skip the base altogether.       }
{  Additional entries will probably be made to include QWK generation.     }
{  The RefID is used by the message bases to hold the reference number of  }
{  the last message read, and used by the Xfer section to store the date   }
{  the section was last scanned (PACKTIME format).  Updated is another     }
{  field which contains a date in PACKTIME, and will contain the date/time }
{  that this record was last updated.  Eventually a utility will be needed }
{  to weed out records which have not been accessed in 90 days or some     }
{  arbitrary time (users who are no longer active on the board).           }
{                                                                          }
{**************************************************************************/

typedef struct
 {
  long               UserId,       /* Unique User Number */
		     RefId;        /* Last message the user read */
  unsigned char      ScanMe;       /* 0 = Dont Scan, 1 = Scan, 2 = Ask */
  long               Updated,      /* Last time this record was used */
		     Space1,       /* unused */
		     Space2;       /* unused */
 } QscanRec;

/**************************************************************************}
{                                                                          }
{  CONFERENCEREC Record                                                    }
{                                                                          }
{  One of Celerity v2.00's central features is its high powered conference }
{  system.  Configuration of the conference system is saved via a series   }
{  of conference data files in the conference directory with an extension  }
{  of .CNF.  There are four basic .CNF files defined by Celerity, all      }
{  others are created by sysops as they set up their systems.  The pre-    }
{  defined conferences are MAIN.CNF, XFER.CNF, PRIVATE.CNF, and GFILE.CNF. }
{  A conference item can be one of a number of types, indicated in the     }
{  DataType field.  The currently defined types are as follows:            }
{  #  Ext   Description                                                    }
{  0        Deleted conference item                                        }
{  1 .BSE   Message Base                                                   }
{  2 .XFR   Transfer Section                                               }
{  3 .CNF   Conference Item                                                }
{  4 .DOR   Doors section                                                  }
{  5 .BUL   Bulletin / news section                                        }
{  6 .ART   Art Gallery                                                    }
{  7 .LST   BBS List                                                       }
{  8 .VOT   Voting Booth                                                   }
{                                                                          }
{**************************************************************************/

typedef struct                      /* *.CNF format for v2.00 */
 {
  char           Name[61],          /* Name of conference item */
		 AccessCode[11],    /* Access Code to select item */
		 ReadAccess[81],    /* Access string required to access */
		 PostAccess[81],    /* Access string required to post */
		 AdminAccess[81],   /* Access String required for sysop access */
		 Password[11],      /* Password to enter sub */
		 DataFile[81],      /* Data item menu entry points to */
		 ConfMenu[81];      /* name of menufile for this item */
  unsigned char  DataType;          /* type of conference item */
  unsigned int   Index;             /* position in data file */
 } ConferenceRec;

/**************************************************************************}
{                                                                          }
{  XFERLISTREC Record                                                      }
{                                                                          }
{  The XferListRec record is used by the XFERLIST.UPL and XFERLIST.DNL     }
{  files residing in the DATA directory.  These files contain logs of      }
{  the last 500 files uploaded and downloaded from the BBS.  The most      }
{  recent upload or download resides at the end of the file, and the oldest}
{  file is on the beginning.  When the file is full (500 entries), the     }
{  oldest 10 entries will be erased and the other entries moved down.      }
{                                                                          }
{**************************************************************************/

typedef struct
 {
  char             Username[31],         /* Name of user who downloaded file */
		   UppedBy[31],          /* Name of user who uploaded file */
		   Filename[31];         /* Name of file downloaded */
  long             Size,                 /* Size of file in bytes */
		   When;                 /* When the file was downloaded in PACKTIME */
  unsigned int     cps;                  /* CPS rate of the xfer */
 } XferListRec;

/**************************************************************************}
{                                                                          }
{  FILEREC Record                                                          }
{                                                                          }
{  The FileRec record (UdRec in 1.42 and previously) is the record used for}
{  all file listings in Celerity v2.00+ transfer sections.  All Celerity   }
{  *.DIR files are made up of this record, and have a corresponding .DES   }
{  data file (CelerityText with MsgDataHeader - see above) for description }
{  data.  The DescOffset and DescSize refer to this .DES file.             }
{                                                                          }
{**************************************************************************/

typedef struct                        /* *.DIR for version 2.0 */
 {
  char               sentby[31];      /* who uploaded the file */
  long               when,            /* when it was uploaded in PACKTIME */
		     whenrated;       /* when file was validated in PACKTIME */
  char               filename[16],    /* DOS filename */
		     dlpw[16],        /* download password */
		     path[51];        /* full pathname pointing to file */
  int                points;          /* cost to download */
  long               filesize;        /* size in bytes */
  char               SystemNote[45];  /* System Messages ie : Viruses, etc */
  long               DescOffset,      /* Offset of description */
		     DescSize,        /* size of description */
		     ExtDescOffset,   /* offset of extended description */
		     ExtDescSize;     /* size of extended description */
  unsigned char      DescLength;      /* length (# of lines) of description */
  long               DateMoved;       /* date file was last moved in PACKTIME */
  char               Unused[15];      /* unused */
  int                downloaded;      /* number of times downloaded */
  unsigned char      newfile,         /* new file? */
		     FreeFile;        /* file is a free file */
  char               progname[21];    /* program name of file */
  unsigned char      ofwhat1,         /* disk x */
		     ofwhat2;         /* of disk y */
  int                ptsgiven;        /* validation pts awarded */
  char               private[31];     /* who file is private for */
 } FileRec;

/**************************************************************************}
{                                                                          }
{  AREAREC Record                                                          }
{                                                                          }
{  The AreaRec record format defines the structure of all *.XFR conference }
{  items (transfer areas).  It is a typed file containing a list of all    }
{  file areas in the transfer conference.                                  }
{                                                                          }
{**************************************************************************/

typedef struct                    /* .XFR Transfer records for v2.00 */
 {
  char            name[81],       /* Name of xfer area */
		  xmodemdir[81],  /* path to files in section */
		  AccessCode[50]; /* Access string */
  unsigned char   unu1,           /* unused */
		  unu2,
		  unu3,
		  unu4,
		  unu5,
		  AskAboutDiz,    /* Allow user to override file_id.diz */
		  ImportDiz,      /* Import file_id.diz if found */
		  ScanAllAreas,   /* scan other areas */
		  ScanThisArea,   /* scan this area when someone uploads */
		  FreeArea,       /* free download area */
		  NoRatio,        /* ignore ratios */
		  slowdrive;      /* is this a slow drive? */
  int             unused;         /* unused */
  unsigned char   upload,         /* allow uploads */
		  download;       /* allow downloads */
  char            dataname[81];   /* path to .DIR data file */
 } arearec;


/**************************************************************************}
{                                                                          }
{  DOORRECORD Record                                                       }
{                                                                          }
{  The DoorRecord record is used for maintaining lists of Celerity's online}
{  programs for users, most commonly called Doors.  All files in the       }
{  conference directory with an extension of .DOR are made up of these     }
{  records.  The main doors section refers to MAINDOOR.DOR in the          }
{  conference directory, all others are created by sysops in the conference}
{  setup.                                                                  }
{                                                                          }
{**************************************************************************/

typedef struct                          /* *.DOR for Celerity v2.0 */
 {
  char            name[31],             /* Name of door */
		  Access[81];           /* Access Code */
  unsigned int    numused;              /* Number of times used */
  char            Commandline[81];      /* commandline to execute the door */
  unsigned char   doortype,             /* 0 - Celerity, 1 = door.sys, 2 = doorinfo.def, 3 = chain.txt */
		  BatchDoor;            /* Door is a batch file */
 } DoorRecord;

/**************************************************************************}
{                                                                          }
{  ONELINERREC Record                                                      }
{                                                                          }
{  The OneLinerRec record is the structure for ONELINER.DAT in the DATA    }
{  directory.                                                              }
{                                                                          }
{**************************************************************************/

typedef struct                          /* ONELINER.DAT for Celerity v2.0 */
 {
  char              author[31],         /* Handle of author */
		    origin[31];         /* Origin line of where the oneliner was written */
  long              when;               /* data oneliner was written */
  char              Line[81];           /* The actual oneliner */
 } OneLinerRec;

/**************************************************************************}
{                                                                          }
{  ARTRECORD Record                                                        }
{                                                                          }
{  The ArtRecord record is the structure for all *.ART files - the data    }
{  for Art Galleries which reside in the conference directory.  The main   }
{  art gallery is named GALLERY.ART, all others are sysop-created.  Note   }
{  that the ArtType variable is defined as follows:                        }
{                                                                          }
{  1: ANSI art                                                             }
{  2: NAPLPS art                                                           }
{  3: RIP art                                                              }
{                                                                          }
{**************************************************************************/

typedef struct                     /* *.ART definition for 2.0 */
 {
  char           author[31],       /* who uploaded the art */
		 origin[31];       /* origin of art */
  long           when;             /* when uploaded in PACKTIME */
  char           filename[13];     /* Filename of the art */
  unsigned char  TimesViewed,      /* times the art has been viewed */
		 ArtType,          /* Type of art */
		 Unused;           /* unused */
  char           Descript[81];     /* Description of art */
 } ArtRecord;

/**************************************************************************}
{                                                                          }
{  NEWSRECORD Record                                                       }
{                                                                          }
{  The NewsRecord record is used to make up the *.BUL bulletin/news files  }
{  in Celerity, which reside in the CONFERENCE directory.  NEWS.BUL is     }
{  the default news file which is accessible from the main menu and will   }
{  be scanned for new entries when a user logs on.  All other .BUL files   }
{  will act as "bulletin sections" and will not be automatically scanned   }
{  when a user logs on.                                                    }
{                                                                          }
{**************************************************************************/

typedef struct                  /* *.BUL structure for 2.00+ */
 {
  char           author[31],    /* Who wrote the news */
		 origin[31];    /* origin of news */
  long           when;          /* when the news was written in PACKTIME */
  char           filename[13];  /* Filename of news */
  unsigned char  Unused[3];     /* unused */
  char           Descript[81],  /* Description of bulletin */
		 Access[81];    /* Access string required to read */
  unsigned char  AlwaysShow;    /* Always show news */
 } NewsRec;

/**************************************************************************}
{                                                                          }
{  LOGREC Record                                                           }
{                                                                          }
{  The LogRec record contains the format for the system logs, which reside }
{  in the DATA directory and are named SYSLOG.x (x being the node number   }
{  of the log).  The log entries are referenced with a two-dimensional     }
{  numbering system - Menu and Subcommand.  These values are listed below. }
{                                                                          }
{**************************************************************************/

typedef struct                     /* SYSLOG.x format */
 {
  int         menu, subcommand;    /* Identifier - see below */
  char        param[41];           /* paramater passed */
  long        when;                /* PACKTIME format of entry time */
 } logrec;


/*      Menu / Subcommand information.  Not all entries are used.  */

/*

Menu 0:
	0  Incoming call at [Param] bps
	1  User: [Param] Logged on
	2  Invalid password for [Param]
	3  Caller Disconnected
	4  System error: [Param]
	5  Attempted to extract invalid file [Param]
	6  User [Param] in login shell
	7  Calling from area code [Param]
	8  Celerity corrupted.  HALT.
	9  BETA Test Time left: [Param]
	10  Security violation extraction: [Param]
	11  [Param]
Menu 1:
	1  Password changed
	2  Tried to chat
	3  Requested chat: [Param]
	4  ** TRANSFERED TO USER [Param] **
	5  Wrong password transfering to [Param]
	6  Changed handle to [Param]
Menu 2:
	2  Deleted the error log
	3  Edited user [Param]
	4  Changed the system time/date
	5  Created a new guided tour
	6  Viewed the system log
	7  Deleted the System Log
	8  Created user [Param]
Menu 3:
	5  Configured Global Newscan
	6  Configured file listings
	7  Changed Colors
	8  Configured newscans
	9  Changed terminal emulation
	10  Entered Gfile Xfer Section
Menu 4:
	1  Created/uploaded news or bulletin [Param]
	2  Listed bulletins in [Param]
	3  Deleted news/bulletin [Param]
	4  Viewed news/bulletin [Param]
Menu 6:
	1  Added Oneliner [Param]
	2  Deleted Oneliner [Param]
	3  Listed Oneliners
	4  Purged Onliners
Menu 7:
	1  Added BBS [Param]
	2  Added VMB [Param]
	3  Added NUA [Param]
	4  Listed BBS''
	5  Listed VMB''s
	6  Listed NUA''s
	7  Deleted entry [Param]
Menu 8:
	1  Entered chat with [Param]
	2  Exited chat
Menu 9:
	1  Opened door [Param]
	2  Changed door [Param]
	3  Deleted door [Param]
	4  Created door [Param]
Menu 10:
	1  Voted on ballot [Param]
	2  Edited ballot [Param]
	3  Deleted ballot [Param]
	4  Created ballot [Param]
	5  Added option to [Param]
Menu 11:
	1  Listed Art files
	2  Deleted art: [Param]
	3  Uploaded art: [Param]
Menu 12:
	1  Read email from [Param]
	2  Sent email to [Param]
	3  Replied to mail from [Param]
	4  Forwarded email to [Param]
	5  Bulk mail to user #[Param]
	6  Sent feedback to [Param]
	7  U/L file mail: [Param]
	8  D/L file mail: [Param]
	9  Packed mail file: [Param]
Menu 15:
	1  Downloaded [Param]
	2  Uploaded [Param]
	3  Entered sysop options for [Param]
	4  Download Efficiency: [Param]
	5  Batch Downloaded [Param]
	6  Xfer Aborted: [Param]
	7  Xfer Aborted just before EOF: [Param]
	8  Master list for Conference [Param]
	9  Extracted a file from: [Param]
	11  Deleted own upload of [Param]
	12  Changed description of [Param]
	13  Edited own upload of [Param]
	14  DSZ String [Param]
Menu 16
	1  Searched for unvalidated files
	2  Killed area
	3  Modified area attributes for [Param]
	4  Modified file attributes for [Param]
	5  Moved file [Param]
	6  Sorted area
	7  Deleted file [Param]
	8  Added resident file [Param]
	9  Re-ordered areas
	10  Disk directory [Param]
	11  Batch moved file [Param]
	12  Created new area: [Param]
	13  Set upload directory to [Param]
	14  Attempted Security Breach.  File: [Param]
	15  Downloaded offline file: [Param]
	16  Bad offline download: [Param]
Menu 18:
	1  Set wanted flag to [Param]
	6  Changed user''s name to [Param]
	7  File transfer points: [Param]
	8  Changed user''s password
	9  Deleted user
	10  Granted sysop status: [Param]
	11  Denied sysop status: [Param]
	12  Time left for today: [Param]
	13  Viewed user''s status
	14  File transfer level: [Param]
	15  Set user''s access level: [Param]
	16  Changed phone number: [Param]
	17  Part-sysop security violation
Menu 19:
	1  Created QWK Offline packet
	2  Uploaded REP Offline packet
	3  Posted via QWK: [Param]
Menu 20:
	1  Entered Multinode Chat
	2  Exited Multinode Chat
	3  Paged node #[Param]
Menu 22:
	1  Voted yes on [Param]
	2  Voted no on [Param]
	3  NUV: Auto-Validated [Param]
	4  NUV: Auto-Deleted [Param]
	5  Removed [Param] from voting
	6  Vetoed vote on [Param]
	7  Abstained on [Param]
Menu 23:
	0  ULP: Good Upload: [Param]
	1  ULP: Upload Altered: [Param]
	2  ULP: New Description: [Param]
	3..5  ULP: <reserved> [Param]
	6  ULP: Validity failed, [Param] deleted.
	7  ULP: Validity failed, [Param] waiting.
	8  UPL: Virus found, [Param] deleted.
	9  UPL: Virus found, [Param] waiting.
	10  UPL: Virus found in [Param], removed.
Menu 24:
	0  Upload Validity failed, [Param] deleted.
	1  Upload found virus, [Param] deleted.
	2  ULP: New Description: [Param]
	3..5  ULP: <reserved> [Param]
	6  ULP: Validity failed, [Param] deleted.
	7  ULP: Validity failed, [Param] waiting.
	8  UPL: Virus found, [Param] deleted.
	9  UPL: Virus found, [Param] waiting.
	10  UPL: Virus found in [Param], removed.

*/


/**************************************************************************}
{                                                                          }
{  PROTOREC Record                                                         }
{                                                                          }
{  The ProtoRec record is the structure for PROTOCOL.DAT in the DATA       }
{  directory.  It contains information about transfer protocols available  }
{  on a Celerity BBS.  There are five classes of protocols (a class of 0   }
{  is ignored).                                                            }
{                                                                          }
{  1:  Batch Uploads                                                       }
{  2:  Batch Downloads                                                     }
{  3:  Single File Upload                                                  }
{  4:  Single File Download                                                }
{  5:  Bidirectional Transfer                                              }
{                                                                          }
{**************************************************************************/

typedef struct                           /* PROTOCOL.DAT file for v2.0 */
 {
  unsigned char         letter;          /* letter to select */
  char                  desc[31],        /* description */
			progname[13],    /* name of protocol.exe */
			Param[61];       /* Command line */
  unsigned char         Class;           /* class, see above */
 } Protorec;

/**************************************************************************}
{                                                                          }
{  SYSTEMHISTORYREC Record                                                 }
{                                                                          }
{  SystemHistoryRec defines the structure for the HISTORY.DAT file which   }
{  resides in the DATA directory.  This maintains the system history for   }
{  a number of days (including the current one) and keeps track of the     }
{  number of calls, posts, feedback, email, new user applications, uploads,}
{  downloads, etc.                                                         }
{                                                                          }
{**************************************************************************/

typedef struct                  /* HISTORY.DAT structure for 2.0 */
 {
  int           calls,          /* Number of calls made this day */
		posts,          /* # of posts */
		feedback,       /* # of feedback */
		email,          /* # of mail sent */
		newusers,       /* # of new users */
		uploads,        /* # of uploads */
		downloads,      /* # of downloads */
		daynumber;      /* # of days system has been up */
  unsigned char netmaid;        /* unused */
  char          date;           /* date in ASCII format */
  int           temp1,          /* unused */
		temp2,
		temp3,
		temp4,
		temp5;
 } SystemHistory;

/**************************************************************************}
{                                                                          }
{  NODEIDREC Record                                                        }
{                                                                          }
{  The NodeIDRec is important for multi-node systems, and is virtually     }
{  unused for single line systems.  It passes information between nodes    }
{  via the NODELIST.NOD file in the MULTINODE directory.  There are a      }
{  number of different activity codes, which are as follows:               }
{                                                                          }
{  0: Unknown                                                              }
{  1: Node Down                                                            }
{  2: Node Waiting For Call                                                }
{  3: User Logging In                                                      }
{  4: User At Main Menu                                                    }
{  5: User In Sub-Boards                                                   }
{  6: User In Transfer Section                                             }
{  7: User Transfering Files                                               }
{  8: User In MultiNode Chat                                               }
{  9: User Chatting With Sysop                                             }
{ 10: New User Applying                                                    }
{ 11: User In QWK Processor                                                }
{ 12: User In A Door                                                       }
{ 13: User Entering System                                                 }
{ 14: User In Direct (Multinode) Chat                                      }
{ 15: User Linking Another System                                          }
{ 16: Network Call                                                         }
{                                                                          }
{**************************************************************************/

typedef struct                             /* NODELIST.NOD for v2.0 */
 {
  char               Username[16],         /* Name of user */
		     NodeDescript[16];     /* Description of node */
  unsigned char      ChatAvail,            /* Available for paging */
		     ChatRequest,          /* Node being paged on */
		     Activity,             /* Node activity (see above) */
		     DirectPipe;           /* Chat room user is in */
  long               Update;               /* last date/time node updated nodelist.nod */
 } NodeIDRec;

/**************************************************************************}
{                                                                          }
{  CALLERLOGREC Record                                                     }
{                                                                          }
{  The CallerLogRec record defines CALLER.LOG in the DATA directory.  This }
{  file contains records for the previous 250 callers to the system with   }
{  the most recent caller at the beginning of the file.                    }
{                                                                          }
{**************************************************************************/

typedef struct                        /* CALLER.LOG for version 2.0 */
 {
  char              UserName[31],     /* name of user */
		    Location[31];     /* unused */
  long              CallNumber,       /* call number */
		    TimeofCall,       /* time/date in Packtime */
		    Speed;            /* Connect speed, 0 = local */
  unsigned char     ConnectMode,      /* unused */
		    NodeNumber;       /* Node # logged onto */
 } CallerLogRec;

/**************************************************************************}
{                                                                          }
{  SYSTEMCALLSREC Record                                                   }
{                                                                          }
{  This record defines the format of STATUS.DAT in the MAIN BBS Directory. }
{  This file is a typed file, with record #0 containing data for the BBS   }
{  as a whole, and successive records being specific to particular nodes   }
{  (ie: record #1 is for node #1, record #5 for node #5, etc.).            }
{                                                                          }
{**************************************************************************/

typedef struct
 {
  long            TotalCalls;              /* Total number of calls this node */
  unsigned int    CallsToday,              /* Calls today */
		  CallsYesterday;          /* calls yesterday */
  long            CallsLocal,              /* total local calls */
		  Calls300,                /* total 300bps calls */
		  Calls1200,               /* total 1200bps calls */
		  Calls2400,               /* total 2400bps calls */
		  Calls4800,               /* total 9600bps calls */
		  Calls9600,               /* total 12000bps calls */
		  Calls12000,
		  Calls14400,
		  Calls16800,
		  Calls19200,
		  Calls21600,
		  Calls24000,
		  Calls26400,
		  Calls28800,
		  Calls38400,
		  Calls57600,
		  Calls115k,               /* Total 115,200 calls */
		  CallsFiler,              /* Cfiler Calls (unused) */
		  CallsNetwork,            /* Network Calls */
		  CallsLink,               /* Link Calls */
		  DateFirstOnline;         /* packtime that this node first came up */
  unsigned int    TotalDaysUp,             /* number of days node has been up */
		  TotalNewUsers;           /* total new users on this node */
  long            TotalUps,                /* Total uploads on this node */
		  TotalDns,                /* total downloads on this node */
		  TotalUpK,                /* total Uploaded K */
		  TotalDnK,                /* total downloaded K */
		  TodaysDateTime,          /* packtime of last day written to file */
		  MinutedIdle,             /* minutes of inactivity */
		  MinutesUsed,             /* minutes used on this node */
		  MinutesXfer,             /* minutes used for xfers */
		  TotalSystemIDs;          /* total system id's */
  unsigned char   SusopAvailable,          /* Sysop is available? */
		  ScreenOutput;            /* screen output is on */
  unsigned int    TotalMessages;
 } SystemCallsRec;

/**************************************************************************}
{                                                                          }
{  NODECOMMANDREC Record                                                   }
{                                                                          }
{  This record will be used for sysops to pass commands to other BBS nodes.}
{  It will contain information to toggle different options on the node or  }
{  to perform actions.  If an entry is blank, it is ignored.  The file     }
{  this record is used for is NODECMD.<node number> in the SHARED dir.     }
{                                                                          }
{**************************************************************************/

typedef struct
 {
  unsigned char   Snoop,
		  Print,
		  Trap,
		  HangUp,
		  TempSysop;
  int             TimeAdjust,
		  PointsAdjust;
 } NodeCommandRec;

/**************************************************************************}
{                                                                          }
{  TEXTINDEXREC Record                                                     }
{                                                                          }
{  This record will is used by CelerityText files to point to the actual   }
{  location in the .LNG/.DSP file of a particular entry.  It contains both }
{  the offset (location) and size (length) of the entry.                   }
{                                                                          }
{**************************************************************************/

typedef struct
 {
  long            Size,                    /* Size of entry */
		  Offset;                  /* Offset in data file */
 } TextIndexRec;

/***************************************************************************}
{                                                                          }
{  LANGREC Record                                                          }
{                                                                          }
{  This record defines the format of the LANGFILE.DAT file which resides in}
{  the LANGUAGE directory.  It is used to define the language files which  }
{  are available on a BBS.                                                 }
{                                                                          }
{**************************************************************************/

typedef struct
 {
  char            Intro[81],               /* Description shown to user */
		  LangPath[81],            /* Path to language file */
		  MenuPath[81],            /* Path to menu file */
		  Archives[5][81],         /* unused */
		  ArchDesc[5][61],         /* unused */
		  AccessCode[81];          /* Access code */
  unsigned char   DisplayLines,            /* Recommended display length */
		  DisplayCols,             /* Recommended display columns */
		  Ascii7,                  /* Does it support 7 bit ASCII? */
		  Ascii8,                  /* Does it support 8 bit ASCII? */
		  ANSI,                    /* Does it require ansi? */
		  Avatar,                  /* Does it require avatar? */
		  Vt52,                    /* Does it require vt52? */
		  Vt52Atari,               /* Does it require vt52 atari? */
		  Vt100,                   /* Does it require vt100? */
		  Iemsi,                   /* Is it an IEMSI session? */
		  Naplps,                  /* Does it require naplps? */
		  Rip,                     /* Does it require RIP? */
		  dummy1,                  /* unused */
		  dummy2,                  /* unused */
		  dummy3,                  /* unused */
		  dummy4,                  /* unused */
		  dummy5;                  /* unused */
 } LangRec;

/**************************************************************************}
{                                                                          }
{  UPLOADSPOOLREC Record                                                   }
{                                                                          }
{  This record defines the format of the SPOOL.UP file residing in the     }
{  COMMON directory.  It is used to spool uploads to an offline upload     }
{  processor.                                                              }
{                                                                          }
{**************************************************************************/

typedef struct
 {
  FileRec         Ud;                      /* File record of the file */
  char            DestDir[256],            /* directory the file will be moved to */
		  DestArea[256];           /* path to .DIR/.DES file */
  long            UploaderID;              /* UserID of uploader */
 } UploadSpoolRec; 

/**************************************************************************}
{                                                                          }
{  BALLOTREC Record                                                        }
{                                                                          }
{  This record defines the format of the *.VOT files residing in the DATA  }
{  directory.  It is used for taking polls of the users.  Actual vote data }
{  is currently stored in the user record with individual users.  This will}
{  be changed in version 2.10.                                             }
{                                                                          }
{**************************************************************************/

typedef struct
 {
  char            Title[81];               /* Title of voting topic */
  unsigned char   ReqBal,                  /* Required ballot voted on */
		  ReqChoice,               /* Required previous ballot choice */
		  AllowAdd;                /* Allow user additions */
  unsigned int    VotesCast;               /* Total votes cast */
  char            Choices[15][31];         /* voting choices on ballot */
 } BallotRec;

/****************************************************************************}
{ NetConfig record -   FileName: CELTOSCN.CFG                                }
{                      Location: Celerity's Network Directory Preferred, but }
{                                Not Required (To be blunt, they *Could* put }
{                                It anywhere, the path is passed via cmdline }
{****************************************************************************/

typedef struct
 {
  int                 NetNumber,                  /* Network number */
		      AddrZone,                   /* Network Zone */
		      AddrNet,                    /* Network Portion of address */
		      AddrNode,                   /* Node portion of address */
		      AddrPoint;                  /* Point portion of address */
  unsigned char       AllowNetMail,               /* Process netmail? */
		      UseExtNL;                   /* Create extended NL? */
  char                NetName[41],                /* Name of network */
		      CelCfgPath[121],            /* Path to config.dat */
		      FDIMMainDir[121],           /* path to Front door */
		      NLPath[121],                /* Path to real nodelist */
		      NLName[13],                 /* Nodelist file name */
		      NMailTossDir[121];          /* Full path to toss netmail to/from */
  unsigned char       array[474];                 /* unused */
 } NetConfig;

/**************************************************************************}
{ NLIdx record -     FileName: CELNLIDX.### - ### = Relative Net Number    }
{                    Location: Celerity's Network Directory Preferred, but }
{                                Not Required (WILL be in same directory as}
{                                NetConfig Record)                         }
{**************************************************************************/

typedef struct
 {
  int                Zone,       /* Beginning of Zone/Network */
		     Net;        /* Combination */
  long               NLOffset;   /* Physical Offset in CT NLData */
 } NLIdx;

/**************************************************************************}
{ NLData record -    FileName: CELNL.### - ### = Relative Net Number       }
{                    Location: Celerity's Network Directory Preferred, but }
{                              Not Required (WILL be in same directory as  }
{                              NetConfig Record)                           }
{                    This Record serves 2 purposes:                        }
{                     -To Validate a Net address                           }
{                     -Record offset to extended Node List                 }
{**************************************************************************/

typedef struct
 {
  int                Zone,        /* ZONE:NET/NODE */
		     Net,         /* Style Address */
		     Node;        /* unused */
  unsigned int       ExtRecNo;    /* Extended NL rec num */
 } NLData;

/****************************************************************************}
{ NLExtended record -  FileName: CELNLEXT.### - ### = Relative Net Number    }
{                      Location: Celerity's Network Directory Preferred, but }
{                                Not Required (WILL be in same directory as  }
{                                NetConfig Record)                           }
{****************************************************************************/

typedef struct 
 {
  int            Status;        /* status : 0=node,1=ZC,2=RC,3=NC,4=hub,5=pvt,
					    6=hold,7=down */
  char           Name[26],      /* Name of this system */
		 Location[16],  /* Location City/State */
		 Sysop[16],     /* Sysops Name */
		 Phone[17];     /* Telephone of -Unpublished- */
  long           BaudRate;      /* Max Baud rate */
  char           Flags[21];     /* Flags (ie. HST, V32, XA, XX, MO, CM) */
 } NLExtended;

/**************************************************************************}
{                                                                          }
{  SETUPTYPE Record                                                        }
{                                                                          }
{  The SetupType record is a very large record which is used to maintain   }
{  the configuration for Celerity BBS.  It is created, edited, and written }
{  by the CELSETUP.EXE program, and is read by CELERITY.EXE.  The data     }
{  defined by this record gets written to SETUP.DAT, a seperate copy in    }
{  each NODE directory.                                                    }
{                                                                          }
{**************************************************************************/

typedef struct                   /* this is the structure of the configurable */
 {                               /* login, it is a structure with in the main */
  unsigned char   order;         /* data record. You MUST include it!         */
  char            name[13];
  unsigned char   options;
 } Sequence;

   /* Record contained in SETUP.DAT for Celerity v2.0 */

typedef struct
 {
  unsigned int    Version;                /* version number 200 */
  long            CheckSum;               /* dummy variable */
  unsigned char   filter;                 /* dummy variable */
  char            SerialNum[6],           /* serial number */
		  sysop_name[81],         /* unused as of */
		  longname[81],           /* full bbs name */
		  shortname[81],          /* short bbs name */
		  acronym[4],             /* 3-letter acronym */
		  bbscity[31],            /* city of BBS */
		  BbsState[31],           /* state/country */
		  telenumber[31],         /* phone number */
		  sys2name[31],           /* name for system 2 */
		  sys3name[31];           /* name for system 3 */
  unsigned char   allowdoors,             /* allow doors? */
		  allowfat,               /* allow FAT file move? */
		  StrictPhone,            /* use strict phone checking? */
		  SyslogProtect,          /* protect system log */
		  ExternalChatCall,       /* use external chat calls */
		  HangUpOnExit,           /* hang up on user on bbs exit */
		  autologin;              /* sysop autologin */
  unsigned int    SupportedRates,         /* Supported BPS rates */
		  systemtype,             /* system type. 0 = celerity, 1 = CAE, 2 = CAE/TAC, 3 = Alacrity */
		  systemspeed;            /* CPU type. 0 = 486, 1 = 386, 2 = 286, 3 = 8086 */
  int             bbsnode,                /* Node number of bbs */
		  mintimeout;             /* minutes of inactivity till time out */
  unsigned char   UserTopTenDisable,      /* allow users to disable top ten */
		  UseTopTen,              /* use top ten */
		  UseRumor,               /* use oneliners */
		  KeyFlash,               /* key flash monitor */
		  FreeDownloads;          /* leech week? */
  char            WaitText[81],           /* "Please wait" string */
		  LocalCodes[81],         /* local area codes in format : 213;714 */
		  AnonymousStr[31],       /* annoymous string */
		  DotChar[2];             /* hidden input character */
  int             DefHelpLevel;           /* default help level for new users */
  unsigned char   uart16550;              /* enable FIFO buffering */
  int             PortDelay,              /* unused */
		  ComPortSelect,          /* ComPort # */
		  UseCom,                 /* Com port */
		  ComInt,                 /* interupt */
		  IRQ,                    /* IRQ */
		  InBuffer,               /* input buffer */
		  OutBuffer;              /* output buffer */
  char            BaseAddr[11];           /* base address of port */
  unsigned int    LockedDTERate;          /* locked DTE rate */
  unsigned char   LockDTERate,            /* lock the DTE rate */
		  OffHookSysop;           /* offhook on local calls? */
  char            ModemSetupStr[81],      /* modem setup string */
		  ModemHangupStr[81],     /* modem hangup string */
		  ModemDialPrefix[81],    /* modem dialing prefix */
		  ModemDialSuffix[81];    /* modem dialing suffix */
  int             ModemType,              /* type of modem */
		  NumRings,               /* number of rings before answering */
		  ConnectDelay;           /* delay before connecting */
  unsigned char   Monochrome,             /* monochrome system? */
		  UseEGA,                 /* use 43/50 line mode */
		  CelerityAnsi,           /* use internal ansi */
		  ScreenSaver;            /* use screen saver */
  int             UploadFactor,           /* grant how many times the point val */
		  PointVal,               /* how many k needed per file pt */
		  Maxloan,                /* maximum loan */
		  Commission,             /* commision rate */
		  ULpercent;              /* time compensation */
  unsigned int    DownLoadRates;          /* baud rates allowed to download */
  unsigned char   UserDLcost,             /* can user decide DL cost? */
		  UseFPS,                 /* use file pt system */
		  AutoValidate,           /* autovalidate uploads */
		  Process,                /* process uploads */
		  UseBackupDir,           /* user backup dirs? */
		  AllowRequest,           /* allow user requests for offline files */
		  Files_Password,         /* allow file passwords */
		  PickyDesc,              /* use picky descriptions */
		  Files_Progname,         /* use program name */
		  Files_OfWhat,           /* use disk numbers */
		  Files_Descript,         /* use descriptions? */
		  UseConf;                /* unused */
  char            ConfNames[5][31],       /* unused */
		  XferConfNames[5][31],   /* unused */
		  sysoppass[11],          /* sysop password */
		  pagepass[11],           /* emergency chat password */
		  systempassword[31],     /* system password */
		  system2password[31],    /* system 2 password */
		  system3password[31],    /* system 3 password */
		  newuserpass[31],        /* new user password */
		  lockoutpass[31],        /* low speed lockout password */
		  AvailStr[81],           /* "Sysop available" string */
		  NotAvailStr[81],        /* "Sysop not available" string */
		  Appear[81],             /* enter chat string */
		  DisAppear[81];          /* exit chat string */
  unsigned char   ObnoxChat;              /* use obnoxious chat */
  char            LoginString[81];        /* login text to be displayed */
  unsigned char   UseTermDetect;          /* attempt to detect emulation */
  unsigned int    ShellType;              /* 0 = no shell, 1 = menu, 2 = VMS shell, 3 = DOS shell, 4 = pulldown */
  char            Main_Dir[128],          /* main bbs dir */
		  Temp_Dir[128],          /* temp. dir */
		  Node_Dir[128],          /* path to node directory */
		  Multi_dir[128],         /* path to common dir. */
		  Conf_Dir[128],          /* conference files */
		  Data_Dir[128],          /* data directory */
		  Display_Dir[128],       /* display directory (text files) */
		  CelNet_Dir[128],        /* Cel Net files */
		  Door_Dir[128],          /* door directory */
		  Audio_Dir[128],         /* audio directory */
		  NUU_Dir[128],           /* New User Uploads */
		  Virus_Path[81],         /* path to virus scanner */
		  Virus_Command[81];      /* Command line for virus scanner */
  unsigned char   UseNet,                 /* use celeritynet */
		  NetHost,                /* net host */
		  NetHub;                 /* net hub */
  char            netnode[7],             /* network node # */
		  netpass[81];            /* password */
  long            hubnum;                 /* hub ID # */
  char            netbegin[9],            /* network begin */
		  netend[9],              /* network end */
		  hostphone[31],          /* host phone number */
		  extender[81],           /* dialing extender */
		  originline[81];         /* origin line */
  unsigned char   FeatureA,               /* network features */
		  FeatureB,
		  FeatureC,
		  FeatureD,
		  FeatureE,
		  FeatureF,
		  FeatureG,
		  FeatureH,
		  FeatureI,
		  FeatureJ;
  char            TimeRestStart[9],       /* restricted login start */
		  TimeRestEnd[9],         /* restricted login end */
		  AvailTime[9],           /* time sysop available */
		  UnavailTime[9],         /* time sysop unavailable */
		  EventTime[9],           /* time to run batch event */
		  EventBatch[9],          /* name of batch event to run */
		  BackTime[9],            /* time to run backup */
		  ShellCom[9][11],        /* shell commands */
		  Infoname[5][31];        /* names of infoforms */
  unsigned char   ForceForm[5];           /* force infoforms */
  int             InfoLvl[5],             /* level to use infoforms */
		  InfoMax[5];             /* max level to use infoforms */
  unsigned char   private,                /* refuse new users */
		  HangNewUsers,           /* hang up after apply */
		  RequireUpload,          /* require new user uploads */
		  CopyNewUserUpload,      /* save new user uploads */
		  RequireFeedback,        /* require new user feedback */
		  UseAddress;             /* ask for address */
  int             NewUserLvl,             /* new user level */
		  DefUDLevel,             /* default xfer level */
		  DefUDpoint;             /* default points */
  char            NewUserNote[31];        /* new user note */
  unsigned char   NewUserVoting,          /* new user voting enable */
		  AutoNUV;                /* autovalidate */
  int             YesVote,                /* number of yes votes needed */
		  NoVote,                 /* number of no votes needed */
		  NuvLvl,                 /* level to use NUV */
		  Forcevote;              /* level to force NUV */
  unsigned char   LocalLockout;           /* use local lockout */
  int             LocalPercent;           /* percent of locals allowed */
  char            LocalMSG[81];           /* message to print for lockout */
  int             QvalLvl,                /* Quick Val level */
		  QvalXfer,               /* quick val xfer level */
		  QvalXP;                 /* quick val xfer points */
  char            QvalNote[31];           /* quick val user note */
  unsigned char   QvalMsgCon[5],          /* unused */
		  QvalXferCon[5],         /* unused */
		  BasicMsgCon[5],         /* unused */
		  BasicXferCon[5];        /* unused */
  int             LogonLvl,               /* level required to access bbs */
		  UserTime[10],           /* user time/day for level */
		  SysopLevel,             /* level for sysop */
		  AnonymousLevel,         /* level to write anoymous */
		  PostLevel,              /* level to post */
		  AutoMsgLvl,             /* level to write automessage */
		  TimeRestLvl,            /* level to logon during restricted hours */
		  ReadAnonLvl,            /* level to read anon. messages */
		  ListUserLvl,            /* level to list message */
		  ListLastLvl,            /* level to last callers */
		  ListXferLvl,            /* level for xfer activity */
		  SyStatLvl,              /* level to list system stats. */
		  SysHistLvl,             /* level to view history */
		  TopTenExempt,           /* exempt from top ten level */
		  LoanLev,                /* level to make a loan */
		  XferPcr,                /* default pcr needed for xfer */
		  PCRExempt,              /* level to exempt from PCR */
		  UDRatio,                /* UD ratio required */
		  UDExempt,               /* level to exempt from UDR */
		  GenRatio,               /* default genratio */
		  GenExempt;              /* level to exempt from GENR */
  unsigned char   Color1,                 /* user colors #1 - 8 */
		  Color2,
		  Color3,
		  Color4,
		  Color5,
		  Color6,
		  Color7,
		  Color8,
		  SysopToolColor,         /* Sysop colors */
		  StatLineColor,
		  SysopToolHigh,
		  WFC_Back,
		  WFC_Text1,
		  WFC_Text2,
		  WFC_Panel1,
		  WFC_Panel2,
		  WFC_Border,
		  PopupColor,
		  LightBarColor;
  char            Art_Dir[128],           /* path to art files */
		  Language_Dir[128],      /* path to language files */
		  netbegin2[9],           /* unused */
		  netend2[9];             /* unused */
  unsigned char   AlacrityMenus,          /* use alacrity structure */
		  AllowMasterFile,        /* allow master file list */
		  SoundEffects,           /* use sound effects? */
		  BozoFilter;             /* usused */
  char            Mail_Dir[128];          /* path to email */
  unsigned char   fidoLocalAttribute,     /* fido options */
		  fidoDelSentMsg,
		  fidoDelTossedMsg,
		  fidoSetCrash,
		  fidoSetPrivate,
		  fidoSetHold,
		  fidoSetLocal,
		  fidoFlag8;              /* usused */
  char            fidoAddress[16],        /* address for Fido */
		  fidoOrigin[81],         /* origin for Fido */
		  connectStr[13][16];     /* modem connect strings */
  unsigned char   UserNoteEdit,           /* allow user to edit note */
		  AllowHandles,           /* allow handles */
		  UseNuaVmb;              /* allow NUA/VMB in bbs list */
  char            ShellPrompt[81],        /* shell prompt format */
		  DSZexe[13],             /* unused */
		  ShellUploadPass[11];    /* unused */
  Sequence        sequence[20];           /* configurable login */
  int             ArtGalleryLvl,          /* level to use art gallery */
		  QwkAccessLvl,           /* level to use QWK */
		  SLevExpired,            /* level to change to when expired */
		  XLevExpired;            /* xfer level to change to when expired */
  unsigned int    HeaderFormat,           /* unused */
		  UnusedLF,               /* unused */
		  UnusedLF2;              /* unused */
  char            PauseString[81],        /* usused */
		  LowSpeedNewUserPass[11];/* password for low speed users */
  unsigned char   filepw;                 /* use file passwords */
  char            ScanDrives[11];         /* drives to scan in WFC */
  unsigned char   AdjustDisplay;          /* adjust display feature? */
  char            NodeName[16],           /* name of node */
		  PKZip_Path[128],        /* path to pkzip */
		  PKunzip_Path[128],      /* path to pkunzip */
		  Arj_Path[128],          /* path to ARJ */
		  Lha_Path[128],          /* path to LHA */
		  Zoo_Path[128],          /* path to ZOO */
		  DDD_Path[128],          /* path to DDD */
		  Pak_Path[128],          /* path to PAK */
		  Arc_Path[128],          /* path to arc.exe */
		  Unarc_Path[128],        /* path to unarc */
		  Swap_Dir[128],          /* swap file dir */
		  Buffer_Dir[128],        /* for slow files to be copied to */
		  Archive_Dir[128],       /* for archive use */
		  GifView[128];           /* for GIF viewer */
  unsigned char   VerifyUpload,           /* verify uploads */
		  ConvertUpload,          /* convert uploads */
		  VirusScan,              /* scan for virii */
		  CommentUpload,          /* comment uploads */
		  CleanUpload,            /* clean uploads */
		  Files_Private,          /* allow private files */
		  Files_ExtDesc,          /* allow extended descriptions */
		  AllowMultipleLogins,    /* allow multiple logins */
		  ExitOnRecycle,          /* board drops to batch file */
		  AllowMailFiles;         /* allow attaching of mail files */
  unsigned int    SerialType,             /* Serial, Digi-Board, Fossil */
		  FreeSpaceToUpload,      /* free space required to upload */
		  PrintUserLog;           /* print user log */
  unsigned char   SecureConsole;          /* use secure console */
  char            SendNUFeedBackToo[31];  /* person to send New User Feedback too */
 } setuptype;
