{$I-}
Program ScanSub;

uses dos,crt;

const
	RW				=	 0+2;	    {Fully Open}
	Locked		= 16+2;	    {Read/Write Denied to other programs}
	ReadOnly	=	32+2;	    {Write Access Denied}
	WriteOnly	=	48+2;	    {Read Access Denied}

	Denied		=	 5;	      {File Locked Error}

var
    StripLF:boolean;
type
    DataPtr = ^byte;

{**************************************************************************}
{                                                                          }
{  MSGDATAHEADER Record                                                    }
{                                                                          }
{  The MsgDataHeader is a very important record, but is never used in of   }
{  itself.  For all CelerityText data entries (messages, file descriptions,}
{  menus, language file entries, email, etc), there is an index file and   }
{  a data file.  The index file is always a typed file (see other sections }
{  for details on these record formats), but the data file is dynamic.     }
{  This is where the MsgDataHeader comes in.  At the very beginning of a   }
{  data segment in a CelerityText data file is a special 9-byte header.    }
{  This header is the MsgDataHeader, which contains some important details }
{  about the data that follows.  The IndexOffset refers back to the index  }
{  file and contains the index number of the record which is associated    }
{  with the data segment.  The size tells Celerity how many bytes are in   }
{  the data segment, and CompType refers to the "compression format",      }
{  which is currently unused.                                              }
{                                                                          }
{**************************************************************************}
type	MsgDataHeader = Record
		  IndexOffset	: longint;	{Location of index entry}
		  Size				: longint;	{Size of message}
		  CompType		: byte;			{Compression type - unused in 2.0}
		  end;


{**************************************************************************}
{                                                                          }
{  MSGINDEXREC Record                                                      }
{                                                                          }
{  The MsgIndexRecord is vital to the Celerity v2.00+ messaging system.    }
{  The *.IDX index files in the message bases and MAIL.IDX file are typed  }
{  files based on this record.  It contains all pertinent information      }
{  regarding messages on a Celerity system.                                }
{                                                                          }
{**************************************************************************}
type MsgIndexRec = record     {*.IDX format for Celerity v2.00}
		 Title		:	string[71];	{Title of the message}
		 Author		:	string[35];   {Handle of message author}
		 AuthorAddr:string[127];	  {This is the address of the author}
     AuthorNote: string[30];  {Authors User Note}
		 LeftTo		: string[35];   {Handle of addressee}
		 LeftAddr	:	string[127];	  {Address of the recipient}
		 ForwardBy	: string[35]; {Handle of the user who forwarded the message}
		 ForwardAddr:string[127];	{This is the address of the fowarder}
		 When			: longint;		  {When message was written in PACKTIME}

		 TimesRead	: word;				{Message read how many times??}
		 Return		: boolean;		  {Return Receipt requested}
		 Anon			: boolean;		  {Anonymous message}
		 FileName	: string[12];   {Name of attached file}

		 Offset		: longint;		  {position of data in data file}
		 Size			: longint;		  {size of message in bytes}

		 {flags....}
		 Deleted		: boolean;		{message is deleted}
     Validated : boolean;     {message validatation state}
     Personal  : boolean;     {

		 {Thread Info}
		 Origin,             		  {Original post of thread}
		 Parent,									{Parent of this message}
     PrevSibling,             {Prev reply to the original message. (Sibling)}
		 NextSibling,						  {Next reply to the original message (sibling)}
		 Child			: longint;		{First reply to THIS post.. eldest "child"}

     MsgRefId  : longint;     {Reference to this post}
     FidoToss,
     Toss2,
     Toss3,                   {Flags for third-party utilities}
     ExtTossed : boolean;     {External utility tossed}

     NetIdent  : string[30];  {Name for network}
     AssortedCrap: array[1..82] of char;
		 end;


{**************************************************************************}
{                                                                          }
{  MSGBASEREC Record                                                       }
{                                                                          }
{  The MsgBaseRec record contains the format for Celerity's message base   }
{  configuration files (*.BSE in the conference directory).  It contains   }
{  all needed information about the actual message base, its configuration,}
{  restrictions, and so forth.                                             }
{                                                                          }
{**************************************************************************}
type MsgBaseRec=record {*.BSE format for Celerity v2.00}
		 Name:string[30];                {Name of sub-board}
		 QwkName:string[15];             {Name used in QWK readers}
		 AccessCode:string[15];          {Command to get to sub board}
     AttachedPath,             {Path to attached files}
		 DataPath,                 {File name of the Sub Data}
     IndexPath,                {File name of the Sub Index}
     QscanPath : pathstr;      {File name of quickscan data}
		 OriginLine:string[80];          {This sub's origin line}
		 BoardMsgLimit:integer;    {Maximum number of messages on sub  }
     CurrentMsgCount:integer;  {every time post made, inc. this}
		 NetType:byte;             {0=local, 1=celnet, 2=fido, 3=mercnet, 4=qwk}
		 NetAddress:string[15];          {network address use}
		 MercName:string[15];            {network address use}
		 BaseNews:string[80];            {path to message base news/intro file}
		 Compression:byte;         {0=uncompressed - default}
     PostSizeLimit:integer;    {0=no limit.  A byte was too small.}
     FileCredit:integer;       {bytes per credit point.      }
                               {If negative, points per post.}
		 RealNames,                {Use real names on sub}
		 AllowAnsi,                {Allow ANSI                    }
		 AllowNaplps,              {Allow NAPLPS                  }
		 AllowAnony,               {Allow Anonymous posts         }
		 AllowFileAttach,          {Allow attached files          }
		 AllowPrivateMsg,          {Allow receiver-only messages  }
		 AllowUpload,              {Allow post uploads            }
		 AllowDnload,              {Allow post downloads          }
     RequireValidate,          {Require Post Validation       }
     TimeStopped,              {Time doesn't count when in sub}
		 option1,
		 option2,
		 option3,
		 option4,
		 option5 :boolean;

     CurMsgRefID: longint;    {Last message id on this sub}
     Index      : longint;    {Pointer to the records loc in data file  }
                              {This needs to be updated when subs change}
     QwkConfNum : Word;
     Space: String[158];   {Spare work space}
		 end;


function strlong (l:longint):string;    {Convert longint to string}
var v:string;
begin
  str (l,v);
  strlong:=v
end;

function now:longint;
var dt:datetime;
    t:word;
    l:longint;
begin
  gettime (dt.hour,dt.min,dt.sec,t);
  getdate (dt.year,dt.month,dt.day,t);
  packtime (dt,l);
  now:=l
end;

function upstring (a:string):string;  {Convert string to all uppercase}
var cnt:word;
begin
	for cnt:=1 to length(a) do
		a[cnt]:=upcase(a[cnt]);
	upstring:=a;
end;


function NetReset(var af; size:word; mode:byte):byte;  {File locking reset}
var
	cnt,
	initmode: byte;
	result	: word;
	f				: file absolute af;
begin
	initmode:=filemode;	{store the original filemode}

	filemode:=mode;			{Set the desired mode}

	if size=0 then		{Branch for Record Size}
		reset(f)
	else
		reset(f,size);

	result:=ioresult;

	cnt:=0;

	while (result=5) and (cnt < 200) do begin	{Approx 2 Seconds...}
		if size=0 then		{Branch for Record Size}
			reset(f)
		else
			reset(f,size);

		result:=ioresult;
		inc(cnt);					{Inc Loop Counter}
		delay(10);				{Delay 1/100 sec }

		end;

	FileMode:=InitMode;		{Restore original FileMode}

	NetReset:=Result;			{Return the result code}

end;

procedure CopyToFile(Size:Longint; Offset:longint; FromFile,ToFile:Pathstr);
var                                {Copy a data segment to a file}
    cnt:longint;
    Numread:word;
    SizeRead:longint;
    status:integer;
    f:File;
    ff:file;
    k:char;
    BufPos,Skipped:word;
    buffer,Buffer2:array[1..4096] of char;
begin
{  writeln('Copying ',size,' from ',fromfile,' to ',tofile,'  Offset:',offset);}
  if size=0 then begin
    exit;
    end;
  assign(f,ToFile);
  rewrite(f,1);
  assign(ff,fromfile);
  status:=netreset(ff,1,RW);
  seek(ff,offset+9);
  SizeRead:=size;
  repeat
    blockread(ff,buffer,sizeof(buffer),numread);
    Skipped:=0;
    if StripLF then begin
      Buffer2:=buffer;
      fillchar(Buffer,sizeof(Buffer),#00);
      BufPos:=0;
      for cnt:=1 to Numread do begin
        if Buffer2[cnt]<>#13 then begin
          inc(BufPos);
          buffer[BufPos]:=Buffer2[cnt];
          end
        else  inc(skipped);
        end;  {numread count}
        numread:=BufPos;
    end;  {strip LF}

    if (numread+skipped)>SizeRead then begin
      BlockWrite(f,buffer,SizeRead-skipped);
      SizeRead:=0;
      end
    else begin
      BlockWrite(f,buffer,numread);
      SizeRead:=SizeRead-NumRead+skipped;
      end;
  until (numread=0) or (sizeread<=0);
  close(f);
  close(ff);
end;


{ This procedure will save message data from an external file.     }
{                                                                  }
{ Save the actual message data to the end of the message data file }
{ IndexLoc needs to be passed in so that the data header can refer back}

procedure SaveMsgDataFromFile(var Index:MsgIndexRec; IndexLoc: longint; SFN, FN: pathstr);
{ Save the actual message data to the end of the message data file }
{ IndexLoc needs to be passed in so that the data header can refer back}

const BuffSize =1024;

var
  sourcefile,
	datafile	    : file;
	header		    : MsgDataHeader;
	Result		    : integer;

	numread       : word;

	buffer	      : array[1..BUFFSIZE] of byte;

  Size,
  Offset,
  ReadBuff      : longint;


begin
	{$I-}
	assign(datafile,FN);
	Result:=NetReset(datafile,1,Locked);
  if result=2 then exit;

  {A little extra delay}
  if result=denied then repeat
    close(DataFile);
    delay(100);
  	Result:=NetReset(datafile,1,Locked);
  until (result<>denied);


	case result of

		0,2:begin                         {No errors, save to EOF}

      if Result=0 then begin

			  offset:=filesize(datafile);		{Get Filesize}
			  seek(datafile,offset);				{Seek to EOF}
			  Index.offset:=offset;         {Pos of the Data in data file}
        end {Result=0}

      else begin
			  Result:=NetReset(datafile,1,Locked);
			  rewrite(Datafile,1);
        close(DataFile);

			  Result:=NetReset(datafile,1,Locked);
			  offset:=filesize(datafile);		{Get Filesize}
  		  seek(datafile,offset);				{Seek to EOF}
			  Index.offset:=offset;         {Pos of the Data in data file}
        end; {result=2}


      assign(SourceFile,SFN);
      result:=NetReset(SourceFile,1,READONLY);
      if result <> 0 then begin                    {If we open read source}
         writeln('Source file locked: ',SFN);
         close(SourceFile);
         close(datafile);
         exit;                                     {Bail out.}
         end;



      Size:=FileSize(SourceFile);                  {Get the size of source}
      Index.Size:=Size;                            {Needs to be returned}

      {Setup the header}
			Header.IndexOffset:=IndexLoc;    {This is the Loc of the INDEX}
      Header.Size:=Size;
			Header.CompType:=0;

			blockwrite(datafile,header,sizeof(header) ); {Write the header block}


      {this loop reads in the sourcefile, and write out to datafile}
			repeat
        If Size > BUFFSIZE then begin
           ReadBuff:=BUFFSIZE;
           Size:=Size - BUFFSIZE;
           end
        else begin
           ReadBuff:=Size;
           Size:=0;
           end;

				blockread(sourcefile,buffer,Readbuff,numread);	{Read from source}
        blockwrite(datafile,buffer,numread);            {Write to datafile}

			until Size=0;

			end;

		5:writeln('File Busy: ',FN);				{File Locked - Repeat??}

    else
        writeln('Error #',result,' writing to ',fn,'.  Please report to author.');
	end; {case}

	close(datafile);
  close(sourcefile);
	{$I+}
end;    {SaveMsgDataFromFile}

procedure SaveMsgData(var Index:MsgIndexRec; IndexLoc: longint; Data:dataPtr; FN: pathstr);
{ Save the actual message data to the end of the message data file }
{ IndexLoc needs to be passed in so that the data header can refer back}
var
	datafile	: file;
	offset		: longint;
	header		: MsgDataHeader;
	Result		: byte;

begin
	assign(datafile,FN);
	Result:=NetReset(datafile,1,Locked);

  if result=2 then begin
    writeln(^M'Creating ',fn,^M);
    readln;
    Result:=NetReset(datafile,1,Locked);
		rewrite(Datafile,1);
    close(DataFile);
    result:=0;
    end;

	case result of

		0:begin
			offset:=filesize(datafile);		{Get Filesize}
			seek(datafile,offset);				{Seek to EOF}

			Index.offset:=offset;         {Pos of the Data in data file}

			With Header do begin
				IndexOffset:=IndexLoc;    {This is the Loc of the INDEX}
				Size:=Index.Size;         {Above needs to be RefNumId}
				CompType:=0;
				end;

			blockwrite(datafile,header,sizeof(header) );

			blockwrite(datafile,data^,index.size);
			end;
		5:writeln('File Busy: ',FN);							{File Locked - Repeat??}

		else begin                                {Create the file first}
    end;
	end; {case}

	close(datafile);
end;


procedure ReadMsgData(var offset:longint;var size:word;var data:dataPtr; FN: pathstr);
var
	datafile: file;
	Header	: MsgDataHeader;
	Result	: byte;

begin
	{$I-}
	assign(datafile, FN );
	Result:=NetReset(datafile,1,RW);
	if Result = 0 then begin
    if offset>=filesize(datafile) then begin
      offset:=0;
      size:=0;
      close(datafile);
      exit;
      end;
		seek(datafile,offset);
		blockread(datafile,Header,Sizeof(Header) );
		blockread(datafile,data^,size);
		end
	else begin
    end;

	close(datafile);
end;

procedure CopyData(Size:word; var Offset:longint; FromFile,ToFile:Pathstr);
var Data:DataPtr;                 {Copy data from one data file to another}
    Dummy:MsgIndexRec;
    IndexLoc:longint;
begin
  getmem(Data,Size);
{  write('Copying ',size,' from ',offset,'/',fromfile);}
  ReadMsgData(Offset,Size,Data,FromFile);
  Dummy.Size:=size;

  SaveMsgData(Dummy, 0, Data, ToFile);
  Offset:=Dummy.OffSet;
{  writeln('  to ',offset,'/',Tofile);
  writeln;}
  FreeMem(data,size);
end;

function ReadMsgRec(offset:longint; var Index:MsgIndexRec; FN: pathstr):integer;
var
	IndexFile	: file of MsgIndexRec;
  TempIndex : MsgIndexRec;
	Result    : byte;
  Size      : longint;
begin
	{$I-}
  begin
	  assign(IndexFile, FN );
	  Result:=NetReset(IndexFile,SizeOf(MsgIndexRec),RW);


	  if (Result<>0) then
		  ReadMsgRec:=-2
	  else begin

      size:=filesize(IndexFile);
      If offset < size then begin {Read request does not exceed filesize}

		    seek(IndexFile,offset);
		    read(IndexFile,TempIndex);

        {Deleted Records will not be returned.}
        If TempIndex.Deleted=false then begin
		       ReadMsgRec:=0;     {Normal Return}
           Index:=TempIndex;
           end
        else begin
           ReadMsgRec:=-1 ;   {Deleted Post}
           Index:=TempIndex;  {Return item anyway... Sometimes needed.}
           end
        end

      else
         ReadMsgRec:=-2;       {Attempt to read past end of file}

		  end;

	  close(IndexFile);
    end;
	{$I+}
end;



{This function returns the size of a Sub-Boards MsgRec file.}
function GetMsgRecSize(FN: pathstr):longint;
var
	IndexFile	  : file of MsgIndexRec;
	Result      : integer;
  Size        : longint;
begin
	{$I-}
 begin
	  assign(IndexFile, FN );
	  Result:=NetReset(IndexFile,SizeOf(MsgIndexRec),RW);

	  if (Result<>0) then
		  GetMsgRecSize:=0
	  else
      GetMsgRecSize:=filesize(IndexFile);

	  close(IndexFile);
    end;
	{$I+}
end; {GetMsgIndexFile}


{**********************************************************************}
{                                                                      }
{  FindMsgRefNumber                                                    }
{                                                                      }
{  Do a binary search of the message reference numbers.                }
{  This is to type out a message by its specific number.               }
{                                                                      }
{  Inputs: RefId:  longint;    The Message Reference ID to look for.   }
{          Area:   MsgBaseRec; The Sub Board Record                    }
{                                                                      }
{  Output: Longint;  - Return the pos of the desired message           }
{          -1 = Deleted Message                                        }
{          -2 = Not found                                              }
{                                                                      }
{**********************************************************************}
Function FindMsgRefNumber(RefID: longint; Area: MsgBaseRec): longint;
var
  Msg   : MsgIndexRec;
  Low,
  Mid,
  High  : longint;

  done  : boolean;
  Result: integer;

begin
  done:=false;
  High:=GetMsgRecSize(Area.IndexPath); {Get size of msgrec file, -1}
  Low :=0;


  While (High >= Low) and (Not Done) do begin
    mid:=trunc( (high+low) div 2 );

    Result:=ReadMsgRec(mid,Msg,Area.IndexPath);
    If result<> -2 then begin
       if RefID > Msg.MsgRefId then
          low:=mid + 1
       else if RefId < Msg.MsgRefId then
          high:=mid - 1
       else
          high:=-1;
       end
    else
       done:=true;  {Error reading index file, bail out.}

    end; {While}


  if done then
     FindMsgRefNumber:=-2       {Error reading the file}
  else begin
     if High = -1 then
        if Msg.Deleted then
           FindMsgRefNumber:=-1  {Record is deleted}
        else
           FindMsgRefNumber:=mid {Found the Record!}
     else
        FindMsgRefNumber:= -1;   {Sorry, Record not found.}
     end;

end; {FindMsgRefNumber}

FUNCTION  SearchForRefNum(RefNum,StartLoc: longint; FN: PathStr):longint;
var
  IndexFile   : File of MsgIndexRec;
  TempMsg     : MsgIndexRec;
  Done        : Boolean;
  Result      : Byte;
  Cnt,
  Size        : Longint;

begin

  Assign(IndexFile, FN);
	Result:=NetReset(IndexFile,SizeOf(MsgIndexRec),RW);
	if (Result<>0) then begin
    SearchForRefNum:=-1;     {Search Failed}
		exit
    end
	else begin
    size:=filesize(IndexFile);
    cnt:=StartLoc;            {The starting Location}
    done:=false;

    {Start scanning from desired location}
    while (cnt < size) and (Not Done) do begin
		   seek(IndexFile,cnt);
		   read(IndexFile,TempMsg);
       If TempMsg.MsgRefId = RefNum then       {Is this the right one?}
          done:=true
       else
          Inc(Cnt);

       end; {while}
		end;
	close(IndexFile);

  If Done then
    SearchForRefNum:=cnt   {Found it!}
  else
    SearchForRefNum:=-1;   {Failed Search}

end; {SearchFor...}


procedure SaveMsgRec(var offset:longint; Index:MsgIndexRec; FN: pathstr);
{	Save the Index Record to a message.  Return the offset used.}
{ The offset is needed to place in the DataHeader for the Data Type}
var
	IndexFile	: file of MsgIndexRec;
	Result    : byte;

begin
	{$I-}
	assign(IndexFile,FN);

	Result:=NetReset(IndexFile,SizeOf(MsgIndexRec),Locked);

	case result of

		0:begin
			if offset = -1 then								{-1 for new message}
         offset:=FileSize(IndexFile);           {Offset for net record}

			seek(IndexFile,offset);										{Seed to Record}
			write(IndexFile,Index);
			end;
		5:writeln('File Busy: ',FN);							{File Locked - Repeat??}
    2:begin
			Result:=NetReset(IndexFile,sizeof(MsgIndexRec),Locked);
			rewrite(Indexfile);											{Create File}
			end;
    else
      writeln('Error #',result,' writing to ',fn,'.  Please report to author.');

	end; {case}

	close(IndexFile);
	{$I+}
end;



FUNCTION SaveMsgBaseRec(var MBR: MsgBaseRec; FN:Pathstr): boolean;
var
  BaseFile  : file of MsgBaseRec;
  Result    : byte;

begin

  Assign(BaseFile, FN);
	Result:=NetReset(BaseFile,SizeOf(MsgBaseRec),Locked);
	if (Result<>0) then
    if Result=2 then begin     {Create the file!}
       Rewrite(BaseFile);
       write(BaseFile,MBR);
       SaveMsgBaseRec:=true;
       end
    else
       SaveMsgBaseRec:=false     {Failed}

	else begin
	   seek(BaseFile,0);
	   write(BaseFile,MBR);
     SaveMsgBaseRec:=true;
		end;

	close(BaseFile);

end; {SaveMsgBaseRec}



{**********************************************************************}
{                                                                      }
{  DeleteMessage                                                       }
{                                                                      }
{  Delete a message.                                                   }
{                                                                      }
{  This procedure is complex due to the need to preserve the thread    }
{  stucture                                                            }
{                                                                      }
{  This Function might belong in MsgUnit                               }
{                                                                      }
{  Inputs: MsgLoc: longint;  Offset in MsgRecord of post to delete     }
{  Output: None                                                        }
{                                                                      }
{**********************************************************************}
procedure DeleteMessage(MsgLoc: longint; Sub: MsgBaseRec; AutoDel:boolean);
var
  Child,
  Parent,
  PrevSib,
  NextSib,
  RefId   : longint;

  DelMsg,
  TempMsg   : MsgIndexRec;

  Done,
  Result    : boolean;

begin

  if ReadMsgRec(MsgLoc,DelMsg,Sub.IndexPath) = 0 then
     RefId:=DelMsg.MsgRefId;

  done:=false;


  {Find the absolute location of the post}
  MsgLoc:=FindMsgRefNumber(RefId,Sub);

  If ReadMsgRec(MsgLoc,DelMsg,Sub.IndexPath) <> 0 then {Load Msg to delete}
     begin
       write('error: ',readmsgrec(msgloc,delmsg,sub.indexpath));
       writeln(^M);
     exit;
     end;




  PrevSib:=-1;
  NextSib:=-1;
  Child  :=-1;
  Parent :=-1;


  If DelMsg.PrevSibling <> -1 then  {Start Search at beginning}
     PrevSib:=SearchForRefNum(DelMsg.PrevSibling,0,Sub.IndexPath);

  If DelMsg.NextSibling <> -1 then  {Start search at next record}
     PrevSib:=SearchForRefNum(DelMsg.NextSibling,MsgLoc+1,Sub.IndexPath);


  {Update the previous message}
  If PrevSib <> -1 then begin
     If ReadMsgRec(PrevSib,TempMsg,Sub.IndexPath) = 0 then begin
        TempMsg.NextSibling:=DelMsg.NextSibling;
        SaveMsgRec(PrevSib,TempMsg,Sub.IndexPath);
        end;
     end;

  {Update the next message}
  If NextSib <> -1 then begin
     If ReadMsgRec(NextSib,TempMsg,Sub.IndexPath) = 0 then begin
        TempMsg.PrevSibling:=DelMsg.PrevSibling;
        SaveMsgRec(NextSib,TempMsg,Sub.IndexPath);
        end;
     end;

  {If there are no siblings, then perserve Child/Parent chain}
  If (PrevSib = -1) and (NextSib = -1) then begin

     If DelMsg.Parent <> -1 then
        Parent:=SearchForRefNum(DelMsg.Parent,0,Sub.IndexPath);
     If DelMsg.Child <> -1 then
        Child:=SearchForRefNum(DelMsg.Child,MsgLoc+1,Sub.IndexPath);

     If Parent <> -1 then begin
       If ReadMsgRec(Parent,TempMsg,Sub.IndexPath) = 0 then begin
          TempMsg.Child:=DelMsg.Child;
          SaveMsgRec(Parent,TempMsg,Sub.IndexPath);
          end;
       end;

     If Child <> -1 then begin
       If ReadMsgRec(Child,TempMsg,Sub.IndexPath) = 0 then begin
          TempMsg.Parent:=DelMsg.Parent;
          SaveMsgRec(Child,TempMsg,Sub.IndexPath);
          end;
       end;

     end; {No Sibs}


  {Updating of Thread Info is all done, now mark post as deleted.}
  DelMsg.Deleted:=true;
  SaveMsgRec(MsgLoc,DelMsg,Sub.IndexPath);


end; {Delete Message}


{**********************************************************************}
{                                                                      }
{  DecNumPosts                                                         }
{                                                                      }
{  Decrement the MsgCount in the Message Base Record.                  }
{                                                                      }
{  Inputs: Count: integer     - Dec by this many                       }
{          Area : MsgBaseRec; - Record to update.                      }
{          FN   : pathstr;    - Pathname of BaseRec file.              }
{  Output: none                                                        }
{                                                                      }
{**********************************************************************}
PROCEDURE DecNumPosts(Num: integer; var Area: MsgBaseRec; FN: pathstr);
var
   result: boolean;

begin
   If Area.CurrentMsgCount > 0 then begin
      Area.CurrentMsgCount:=Area.CurrentMsgcount - Num;
      result:=SaveMsgBaseRec(Area, FN);
      end;
end; {DecNumPosts}


{**********************************************************************}
{                                                                      }
{  AutoDelete                                                          }
{                                                                      }
{  Delete the first active message.  Only one.  Maintain BoardMsgLimit }
{                                                                      }
{                                                                      }
{  Inputs: Num  : integer;    - How many to delete                     }
{          Area : MsgBaseRec; - Record to update.                      }
{          FN   : pathstr;    - Pathname of BaseRec file.              }
{  Output: None                                                        }
{                                                                      }
{**********************************************************************}
procedure AutoDelete(Num: integer; var Area: MsgBaseRec; FN: PathStr);
var
  done    : boolean;
  TempRec : MsgIndexRec;
  recnum,
  cnt,
  size    : longint;

begin
  done  :=false;
  cnt   :=Num;
  RecNum:=0;
  size  :=GetMsgRecSize(Area.IndexPath);  {Size of the Message Index}
  fillchar(TempRec,SizeOf(TempRec),#0);

  {It may be faster to read the directly.  If slow, do that}
  while (Cnt > 0) and (RecNum < Size) and (not done) do begin

    if ReadMsgRec(RecNum, TempRec, Area.IndexPath)=-2  then
       done:=true;                    {bail out, bad read}

    if TempRec.Deleted = False then begin  {Mark the message deleted}
       DeleteMessage(RecNum, Area,True);

       Dec(cnt);
       end
    else
       inc(RecNum);

    end; {while not done}

  DecNumPosts(Num-Cnt,Area, FN); {Dec by number of post deleted}

end; {AutoDelete}



Procedure DoPackBase(NameOfBase:string;DeleteOld:integer);  {Pack an index}
var
  NewIndex,
	OldIndex     : file of MsgIndexRec;
	OldData,NewData     : file;
	x,
  numitems  :longint;
	irec      : MsgIndexRec;
  total,
  deleted   :word;
  result    :byte;
  D,N,E     :string;
  OldDAttr,
  OldIAttr  :Word;

	procedure ReadIndex(num: longint; var base: MsgIndexRec);
	begin
		Result:=NetReset(OldIndex,sizeof(MsgIndexRec),2);
		seek(OldIndex,num);
		read(OldIndex,base);
		close(OldIndex);
	end;


	begin
    total:=0;
    deleted:=0;
    fsplit(NameOfBase,D,N,E);
		assign(OldData,d+n+'.dat');
  	assign(OldIndex,d+n+'.IDX');
    GetFAttr(OldData,OldDAttr);
    GetFAttr(OldData,OldIAttr);
		assign(NewData,d+'temp$.dat');
		assign(NewIndex,d+'temp$.idx');
    erase(NewData);
    x:=ioresult;
    erase(NewIndex);
    x:=ioresult;

    result:=NetReset(OldIndex,sizeof(MsgIndexRec),18);
    numitems:=filesize(OldIndex);
		close(OldIndex);

    if numitems<4 then begin
      exit;
      end;

		Result:=NetReset(OldData,1, 18);
    close(OldData);

		rewrite(NewData,1);
		rewrite(NewIndex);
    close(NewData);

    total:=numitems;
		for x:=0 to numitems-1 do begin
      ReadIndex(x,irec);
      gotoxy(1,1);
      write(x,' Reindexing ',irec.title);
      clreol;
			if (not irec.deleted) and (irec.leftto<>'') and (irec.Author<>'') and
        ((DeleteOld<1) ) then begin
        CopyData(irec.size,irec.Offset,d+n+'.dat',d+'temp$.dat');
				write(Newindex,irec);
				end
			else begin
				inc(deleted);
				end;
			end;


    close(OldIndex);
    close(OldData);
		erase(OldData);
    erase(OldIndex);
    close(NewIndex);
    close(NewData);

		rename(NewIndex,d+n+'.idx');
		rename(NewData,d+n+'.dat');

    reset(NewData,1);
    close(NewData);

    SetFAttr(NewIndex,OldIAttr);
    SetFAttr(NewData,OldDAttr);
    x:=ioresult;
    Gotoxy(1,19);
    writeln(N+'.IDX: Items Erased : ',deleted,' from a total of ',total,'   ',
         total-deleted,' remaining.');
	end;




procedure ShowHelp;
begin
  writeln('Syntax: SCANSUB.EXE [path to .IDX]');
  writeln;
  writeln('SCANSUB.EXE is a basic utility for Celerity v2.00 with source code designed');
  writeln('to work as a basis for a message export utility.  SCANSUB.EXE should');
  writeln('be given the path of a Celerity v2.00 message base index file, and it will');
  writeln('then go through and extract all posts which have not yet been exported (as');
  writeln('defined by the ExtTossed flag in each message).  All posts will be extracted');
  writeln('to ASCII text files named sequentially 1.TXT, 2.TXT, 3.TXT, and so forth.');
  writeln;
  writeln;
  writeln('It is hoped that this code will be used by other programmers to develop utilities');
  writeln('for Celerity to import/export messages to a wide variety of network and mail');
  writeln('systems.  Note that the Toss1..Toss3 variables in the msgindexrec can be used');
  writeln('by these utilities in place of ExtTossed.  Some suggested applications include:');
  writeln;
  writeln('* Import/Export of internet mail / UUCP news feeds (possibly with UUPC)');
  writeln('* Fido Echomail / NetMail');
  writeln('* WWIVNet / Link networking');
  writeln('* RIME / PCRelay networking');
  writeln('* Interface to cc:mail or other network email systems');
  writeln('* CelerityNet v1.42 networking');
  writeln('* FelonyNet or any other proprietary network');
end;


var s:string;
    IndexFile:File of MsgIndexRec;
    MsgIndex:MsgIndexRec;
    BaseFile:File of MsgBaseRec;
    MessageBase:MsgBaseRec;
    Cnt,EXtFile:longint;
    P,N,E:string;
    f:file;
    Result,num:integer; {results of reset operations}
    Offset:longint;
begin
  if paramcount<1 then begin     {If no parameters, show help and exit}
    ShowHelp;
    exit;
    end;


  writeln('SCANSUB.EXE v1.1');

  if upstring(paramstr(2))='/NOLF' then  {Enable linefeed stripping}
    StripLF:=true
  else
    StripLF:=false;

  s:=paramstr(1);                {Get path to index file}
  if pos('.IDX',upstring(s))=0 then
    s:=s+'.IDX';
  {$I-}
  extfile:=0;                    {Seed the name for the export files}
  fsplit(S,P,N,E);               {Split path into components}

  assign(BaseFile,P+N+'.BSE');   {Assign and open the message base itself}

  result:=NetReset(BaseFile,SizeoF(MessageBase),RW);
  if ioresult<>0 then begin
    writeln('Error opening .BSE');
    end
  else begin
    read(BaseFile,MessageBase);
    end;
  close(BaseFile);



  assign(IndexFile,s);           {Assign and open the index}
  result:=NetReset(IndexFile,sizeof(MsgIndex),RW);
  if result<>0 then begin
    writeln('Error reading .IDX');
    exit;
    end;


  cnt:=-1;
  while cnt<filesize(IndexFile)-1 do begin  {Seek to all messages}
    inc(cnt);
    seek(IndexFile,cnt);
    read(IndexFile,MsgIndex);               {Read an index}
    write('Message: ',cnt,' of ',filesize(IndexFile),'  '); {Display status}
    if MsgIndex.Deleted then write('Deleted  ');
    if MsgIndex.ExtTossed then write('Exported  ');
    if MsgIndex.ExtTossed then write('Validated  ');
    writeln;

    if not (MsgIndex.Deleted or MsgIndex.validated or MsgIndex.ExtTossed) then begin
      inc(ExtFile);      {Export this message}
      writeln('Exporting message to ',extfile,'.TXT');
      CopyToFile(MsgIndex.Size,MsgIndex.Offset,P+N+'.DAT',strlong(ExtFile)+'.TXT');
      MsgIndex.ExtTossed:=true;  {Change its status to tossed}
      seek(IndexFile,cnt);       {seek to its position in the file}
      write(IndexFile,MsgIndex); {Write the record back}
      end;
  end;

  writeln('Importing import.txt');




  {Ok, let's import an example message now... from "IMPORT.TXT"}

  assign(f,'import.txt');
  result:=NetReset(f,1,ReadOnly);
  if result<>0 then begin  {file can't be opened}
    close(f);
    exit;
    end;
  close(f);

  {the stuff below is arbitrarily defined.  You'll get this out of whatever
  it is you are importing, of course.. this is just for example}

  fillchar(msgindex,sizeof(msgindex),#0); {Zero the message record entry}
  msgindex.Title:='Imported Message';
  msgindex.Author:='<SCANSUB.EXE>';
  msgindex.AuthorNote:='External Program';
  msgindex.LeftTo:='Everyone';
  msgindex.Anon:=false;
	msgindex.TimesRead:=0; {New message, never read}

  msgindex.LeftAddr:='';	{Different for different nets}

 { msgindex.AuthorAddr:=setup.netnode; }

  msgindex.when:=now;
  msgindex.deleted:=false;
  msgindex.TimesRead:=0;
  Msgindex.ExtTossed:=true;  {It wasn't tossed, but was imported.. and we don't
                              want to export this message back out}


  {Get Message Size}
  result:=NetReset(f,1,ReadOnly);
  msgindex.Size:=FileSize(f);
  close(f);


  Offset:=filesize(IndexFile);         {Choose location of new message}
(*
  seek(IndexFile,offset);              {seek to end of file}
  write(IndexFile,MsgIndex);           {Write the record}
*)

  result:=NetReset(BaseFile,SizeOf(MessageBase),RW);     {Open the basefile to get a new ID}
  if ioresult<>0 then begin
    writeln('Error opening .BSE');
    end
  else begin
    read(BaseFile,MessageBase);
    inc(MessageBase.CurMsgRefId);      {Use last ID plus 1}
    inc(MessageBase.CurrentMsgCount);      {Use last ID plus 1}
    seek(BaseFile,0);
    write(BaseFile,MessageBase);       {Write it back}
    end;
  close(BaseFile);


  {Delete first message if too many messages on the sub}
  If MessageBase.CurrentMsgCount > MessageBase.BoardMsgLimit then begin
     Num:=MEssageBase.CurrentMsgCount - MessageBase.BoardMsgLimit;
     AutoDelete(Num,MessageBase,P+N+'.DAT');
     end;




  msgindex.msgRefId:=MessageBase.CurMsgRefId;

  {The following is thread info.. not much use for imported messages}
  msgindex.Origin  :=msgindex.MsgRefId;    {Original Post - Refer to itself}
  msgindex.Parent  := -1;             { -1 to indicate none }
  msgindex.NextSibling := -1;
  msgindex.Child   := -1;


  SaveMsgDataFromFile(msgindex,Offset,'import.txt',P+N+'.DAT'); {Save the raw data}
  seek(IndexFile,offset);              {seek to the record's position}
  write(IndexFile,MsgIndex);           {Write the record}
  close(IndexFile);

  {Some notes on writing messages:  Defining the fields is fairly obvious
   from their titles, but make sure you define most of them.
   We seek to the end of the message index file to place the new message on
   the end of the base.  We then need to refer to the .BSE file to get the
   reference ID of the last message posted on this base.  Increment it by one,
   and use this as your new ID.  We make sure that this new ID is saved back
   to the base file so the next message doesn't get the same ID.
   We then define the thread info, which is all moot as this is an imported
   message and doesn't have connections to anything else.

   The SaveMsgDataFromFile takes the MsgIndex, the offset (meaning the location
   in the index file), the file to import from, and the file to save the data
   to, and imports the message data.  We then write the index back to the
   index file and close it.

   When importing multiple messages, you will do this process for each message
   you import, except you will obviously import name and address information
   as well.}



{  Version 1.1: Added the message deletion code to keep bases to their limited
   sizes.  Deletion code includes thread updating.
   }


end.
